from __future__ import absolute_import
#!/usr/bin/env python

import numpy as np
import numpy.random as nr
from numpy.testing import(assert_array_equal, assert_array_almost_equal,
                          assert_almost_equal)
from nose.tools import assert_true, assert_equal

from ..graph import (WeightedGraph, complete_graph, mst, knn, eps_nn, 
                     wgraph_from_adjacency, wgraph_from_coo_matrix, 
                     concatenate_graphs, wgraph_from_3d_grid)


def basicdata():
    x = np.array( [[- 1.998, - 2.024], [- 0.117, - 1.010], [1.099, - 0.057],
                   [ 1.729, - 0.252], [1.003, - 0.021], [1.703, - 0.739],
                   [- 0.557, 1.382],[- 1.200, - 0.446],[- 0.331, - 0.256],
                   [- 0.800, - 1.584]])
    return x


def basic_graph():
    l = np.linspace(0, 2 * np.pi, 20, endpoint=False)
    x = np.column_stack((np.cos(l), np.sin(l)))
    G = knn(x, 2)
    return G


def basic_graph_2():
    l = np.linspace(0, 2 * np.pi, 20, endpoint=False)
    x = np.column_stack((np.cos(l), np.sin(l)))
    G = knn(x, 2)
    return G, x


def test_complete():
    v = 10
    G = complete_graph(v)
    a = G.get_edges()[:, 0]
    b = G.get_edges()[:, 1]
    inds = np.indices((v, v)).reshape( (2, v * v) )
    assert_array_equal(inds, (a, b))


def test_knn_1():
    x = basicdata()
    G = knn(x, 1)
    A = G.get_edges()[:, 0]
    assert_equal(np.shape(A)[0], 14)

    
def test_set_euclidian():
    G, x = basic_graph_2()
    d = G.weights
    G.set_euclidian(x / 10)
    D = G.weights
    assert_true(np.allclose(D, d / 10, 1e-7))


def test_set_gaussian():
    G, x = basic_graph_2()
    d = G.weights
    G.set_gaussian(x, 1.0)
    D = G.weights
    assert_true(np.allclose(D, np.exp(- d * d / 2), 1e-7))


def test_set_gaussian_2():
    G, x = basic_graph_2()
    d = G.weights
    G.set_gaussian(x)
    D = G.weights
    sigma = np.sum(d * d) / len(d)
    assert_true(np.allclose(D, np.exp(-d * d / (2 * sigma)), 1e-7))
    

def test_eps_1():
    x = basicdata()
    G = eps_nn(x, 1.)
    D = G.weights
    assert_equal(np.size(D), 16)
    assert_true((D < 1).all())

    
def test_mst_1():
    x = basicdata()
    G = mst(x)
    D = G.weights
    assert_equal(np.size(D), 18)


def test_3d_grid():
    """test the 6nn graph
    """
    x0 = np.array([0, 0, 0])
    x1 = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1], [-1, 0, 0], [0, -1, 0],
                   [0, 0, -1]])
    x2 = np.array([[1, 1, 0], [0, 1, 1], [1, 0, 1], [1, -1, 0], [0, 1, -1],
                   [1, 0, -1], [-1, 1, 0], [0, -1, 1], [-1, 0, 1], 
                   [-1, -1, 0], [-1, 0, -1], [0, -1, -1]])
    x3 = np.array([[1, 1, 1], [1, 1, -1], [1, -1, 1], [1, -1, -1],
                   [-1, 1, 1], [-1, 1, -1], [-1, -1, 1], [-1, -1, -1]])
    for x in x1:
        xyz = np.vstack((x0, x))
        assert_equal(wgraph_from_3d_grid(xyz, 6).E, 2)
        assert_equal(wgraph_from_3d_grid(xyz, 18).E, 2)
        assert_equal(wgraph_from_3d_grid(xyz, 26).E, 2)
        for x in x2:
            xyz = np.vstack((x0, x))
            assert_equal(wgraph_from_3d_grid(xyz, 6).E, 0)
            assert_equal(wgraph_from_3d_grid(xyz, 18).E, 2)
            assert_equal(wgraph_from_3d_grid(xyz, 26).E, 2)
        for x in x3:
            xyz = np.vstack((x0, x))
            assert_equal(wgraph_from_3d_grid(xyz, 6).E, 0)
            assert_equal(wgraph_from_3d_grid(xyz, 18).E, 0)
            assert_equal(wgraph_from_3d_grid(xyz, 26).E, 2)
            

def test_grid_3d_1():
    """ Test the 6 nn graphs on 3d grid
    """
    nx, ny, nz = 9, 6, 1
    xyz = np.mgrid[0:nx, 0:ny, 0:nz]
    xyz = np.reshape(xyz, (3, nx * ny * nz)).T
    G = wgraph_from_3d_grid(xyz, 6)
    assert_equal(G.E, 186)

    
def test_grid_3d_2():
    """ Test the 18-nn graph on a 3d grid
    """
    nx, ny, nz = 9, 6, 1
    xyz = np.mgrid[0:nx, 0:ny, 0:nz]
    xyz = np.reshape(xyz,(3, nx * ny * nz)).T
    G = wgraph_from_3d_grid(xyz, 18)
    assert_equal(G.E, 346)
        

def test_grid_3d_3():
    """ Test the 26-nn graph on a 3d grid
    """
    nx, ny, nz = 9, 6, 1
    xyz = np.mgrid[0:nx, 0:ny, 0:nz]
    xyz = np.reshape(xyz,(3, nx * ny * nz)).T
    G = wgraph_from_3d_grid(xyz, 26)
    assert_equal(G.E, 346)


def test_grid_3d_4():
    nx, ny, nz = 10, 10, 10
    xyz = np.reshape(np.indices((nx, ny, nz)), (3, nx * ny * nz)).T
    G = wgraph_from_3d_grid(xyz, 26)
    D = G.weights
    # 6 * 9 * 10 * 10
    assert_equal(sum(D == 1), 5400 )
    # 26 * 8 ** 3 + 6 * 8 ** 2 * 17 + 12 * 8 * 11 + 8 * 7 
    assert_equal(np.size(D), 20952 )
    # 18 * 8 ** 3 + 6 * 8 ** 2 * 13 + 12 * 8 * 9 + 8 * 6
    assert_equal(sum(D < 1.5), 15120)


def test_grid_3d_5():
    nx, ny, nz = 5, 5, 5
    xyz = np.reshape(np.indices((nx, ny, nz)), (3, nx * ny * nz)).T
    G = wgraph_from_3d_grid(xyz, 26)
    D = G.weights.copy()
    G.set_euclidian(xyz)
    assert_array_almost_equal(G.weights, D)


def test_grid_3d_6():
    nx, ny, nz = 5, 5, 5
    xyz = np.reshape(np.indices((nx, ny, nz)), (3, nx * ny * nz)).T
    adj = wgraph_from_3d_grid(xyz, 26).to_coo_matrix().tolil()
    assert_equal(len(adj.rows[63]), 26)
    for i in [62, 64, 58, 68, 38, 88, 57, 67, 37, 87, 59, 69, 39, 89, 33, 
              83, 43, 93, 32, 82, 42, 92, 34, 84, 44, 94]:
        assert_true(i in adj.rows[63])


def test_grid_3d_7():
    """ Check that the grid graph is symmetric
    """
    xyz = np.array(np.where(np.random.rand(5, 5, 5) > 0.5)).T
    adj = wgraph_from_3d_grid(xyz, 6).to_coo_matrix()
    assert_equal((adj - adj.T).nnz, 0)
    adj = wgraph_from_3d_grid(xyz, 18).to_coo_matrix()
    assert_equal((adj - adj.T).nnz, 0)
    adj = wgraph_from_3d_grid(xyz, 26).to_coo_matrix()
    assert_equal((adj - adj.T).nnz, 0)
        
    
def test_cut_redundancies():
    G = basic_graph()
    e = G.E
    edges = G.get_edges()
    weights = G.weights
    G.E = 2 * G.E
    G.edges = np.concatenate((edges, edges))
    G.weights = np.concatenate((weights, weights))
    K = G.cut_redundancies()
    assert_equal(K.E, e)


def test_degrees():
    G = basic_graph()
    (r, l) = G.degrees()
    assert_true((r == 2).all())
    assert_true((l == 2).all())


def test_normalize():
    G = basic_graph()
    G.normalize()
    M = G.to_coo_matrix()
    sM = np.array(M.sum(1)).ravel()
    assert_true((np.abs(sM - 1) < 1.e-7).all())
    

def test_normalize_2():
    G = basic_graph()
    G.normalize(0)
    M = G.to_coo_matrix()
    sM = np.array(M.sum(1)).ravel()
    assert_true((np.abs(sM - 1) < 1.e-7).all())
    

def test_normalize_3():
    G = basic_graph()
    G.normalize(1)
    M = G.to_coo_matrix()
    sM = np.array(M.sum(0)).ravel()
    assert_true((np.abs(sM - 1) < 1.e-7).all())
    

def test_adjacency():
    G = basic_graph()
    M = G.to_coo_matrix()
    assert_true(( M.diagonal() == 0 ).all())
    A = M.toarray()
    assert_true(( np.diag(A, 1) != 0 ).all())
    assert_true(( np.diag(A, -1) != 0 ).all())       


def test_cc():
    G = basic_graph()
    l = G.cc()
    L = np.array(l==0)
    assert_true(L.all())

    
def test_isconnected():
    G = basic_graph()
    assert_true(G.is_connected())
    

def test_main_cc():
    x = basicdata()
    G = knn(x, 1)
    l = G.cc()
    l = G.main_cc()
    assert_equal(np.size(l), 6)

def test_dijkstra():
    """ Test dijkstra's algorithm
    """
    G = basic_graph()
    l = G.dijkstra(0)
    assert_true(np.abs(l[10] - 20 * np.sin(np.pi / 20)) < 1.e-7)

def test_dijkstra_multiseed():
    """ Test dijkstra's algorithm, multi_seed version
    """
    G = basic_graph()
    l = G.dijkstra([0, 1])
    assert_true(np.abs(l[10] - 18 * np.sin(np.pi / 20)) < 1.e-7)


def test_dijkstra2():
    """ Test dijkstra's algorithm, API detail
    """
    G = basic_graph()
    l = G.dijkstra()
    assert_true(np.abs(l[10] - 20 * np.sin(np.pi / 20)) < 1.e-7)


def test_compact_representation():
    """ Test that the compact representation of the graph is indeed correct
    """
    G = basic_graph()
    idx, ne, we = G.compact_neighb()
    assert_equal(len(idx), 21)
    assert_equal(idx[0], 0)
    assert_equal(idx[20], G.E)
    assert_equal(len(ne), G.E)
    assert_equal(len(we), G.E)


def test_floyd_1():
    """ Test Floyd's algo without seed
    """
    G = basic_graph()
    l = G.floyd()
    for i in range(10):
        plop = np.abs(np.diag(l, i) - 2 * i * np.sin(2 * np.pi / 40))
        assert_true(plop.max() < 1.e-4)
        
def test_floyd_2():
    """ Test Floyd's algo, with seed
    """
    G = basic_graph()
    seeds = np.array([0,10])
    l = G.floyd(seeds)
    
    for i in range(10):
        plop = np.abs(l[0, i] - 2 * i * np.sin(2 * np.pi / 40))
        assert_true(plop.max() < 1.e-4)
        plop = np.abs(l[0,19 - i] - 2 * (i + 1) * np.sin(2 * np.pi / 40))
        assert_true(plop.max() < 1.e-4)

    for i in range(10):
        plop = np.abs(l[1, i] - 2 * (10 - i) * np.sin(2 * np.pi / 40))
        assert_true(plop.max() < 1.e-4)
        plop = np.abs(l[1, 19 - i] - 2 * (9 - i) * np.sin(2 * np.pi / 40))
        assert_true(plop.max() < 1.e-4)
  
def test_symmeterize():
    a = np.array([0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6])
    b = np.array([1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 0, 0, 1])
    edges = np.vstack((a, b)).T
    d = np.ones(14)
    G = WeightedGraph(7, edges, d)
    G.symmeterize()
    d = G.weights
    assert_true((d == 0.5).all())


def test_voronoi():
    """ test voronoi labelling with 2 seeds
    """
    a = np.array([0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6])
    b = np.array([1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 0, 0, 1])
    d = np.array([1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2]);
    edges = np.transpose(np.vstack((a, b)))
    G = WeightedGraph(7, edges,d)
    G.symmeterize()
    seed = np.array([0, 6])
    label = G.voronoi_labelling(seed)
    assert_equal(label[1], 0)
    
    
def test_voronoi2():
    """ test voronoi labelling with one seed
    """
    a = np.array([0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6])
    b = np.array([1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 0, 0, 1])
    d = np.array([1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2]);
    edges = np.vstack((a, b)).T
    G = WeightedGraph(7, edges,d)
    G.symmeterize()
    seed = np.array([0])
    label = G.voronoi_labelling(seed)
    assert_equal(label[4], 0)
 

def test_voronoi3():
    """ test voronoi labelling with non-connected components
    """
    a = np.array([0, 1, 2, 5, 6])
    b = np.array([1, 2, 3, 6, 0])
    d = np.array([1, 1, 1, 1, 1]);
    edges = np.vstack((a, b)).T
    G = WeightedGraph(7, edges,d)
    G.symmeterize()
    seed = np.array([0])
    label = G.voronoi_labelling(seed)
    assert_equal(label[4], - 1)

def test_concatenate1(n=10):
    x1 = nr.randn(n, 2) 
    x2 = nr.randn(n, 2) 
    G1 = knn(x1, 5)
    G2 = knn(x2, 5) 
    G = concatenate_graphs(G1, G2)
    assert_true(G.cc().max() > 0)


def test_concatenate2(n=10):
    G1 = complete_graph(n)
    G2 = complete_graph(n)
    G = concatenate_graphs(G1, G2)
    assert_true(G.cc().max() == 1)


def test_anti_symmeterize():
    n = 10
    eps = 1.e-7
    M = (nr.rand(n, n) > 0.7).astype(np.float) 
    C = M - M.T
    G = wgraph_from_adjacency(M)
    G.anti_symmeterize()
    A = G.to_coo_matrix()
    assert_true(np.sum(C - A) ** 2 < eps)


def test_subgraph_1(n=10):
    x = nr.randn(n, 2) 
    G = WeightedGraph(x.shape[0])
    valid = np.zeros(n)
    assert(G.subgraph(valid) is None)


def test_subgraph_2(n=10):
    x = nr.randn(n, 2) 
    G = knn(x, 5)
    valid = np.zeros(n)
    valid[:n // 2] = 1
    assert_true(G.subgraph(valid).edges.max() < n / 2)


def test_graph_create_from_array():
    """Test the creation of a graph from a sparse coo_matrix 
    """
    a = np.random.randn(5, 5)
    wg = wgraph_from_adjacency(a)
    b = wg.to_coo_matrix()
    assert_array_equal(a, b.todense())


def test_graph_create_from_coo_matrix():
    """Test the creation of a graph from a sparse coo_matrix 
    """
    import scipy.sparse as spp
    a = (np.random.randn(5, 5) > .8).astype(np.float)
    s = spp.coo_matrix(a)
    wg = wgraph_from_coo_matrix(s)
    b = wg.to_coo_matrix()
    assert_array_equal(b.todense(), a)


def test_to_coo_matrix():
    """ Test the generation of a sparse matrix as output 
    """
    a = (np.random.randn(5, 5)>.8).astype(np.float)
    wg = wgraph_from_adjacency(a)
    b = wg.to_coo_matrix().todense()
    assert_array_equal(a, b)


def test_list_neighbours():
    """ test the generation of neighbours list
    """
    bg = basic_graph()
    nl = bg.list_of_neighbors()
    assert_equal(len(nl), bg.V)
    for ni in nl:
        assert_equal(len(ni), 2)


def test_kruskal():
    """ test Kruskal's algor to thin the graph
    """
    x = basicdata()
    dmax = np.sqrt((x ** 2).sum())
    m = mst(x)
    g = eps_nn(x, dmax)
    k = g.kruskal()
    assert_almost_equal(k.weights.sum(), m.weights.sum())


def test_concatenate3():
    """ test the graph concatenation utlitity
    """
    bg = basic_graph()
    cg = concatenate_graphs(bg, bg)
    valid = np.zeros(cg.V)
    valid[:bg.V] = 1
    sg = cg.subgraph(valid)
    assert_array_equal(sg.edges,  bg.edges)
    assert_array_equal(sg.weights, bg.weights)


def test_cliques():
    """ test the computation of cliques
    """
    x = np.random.rand(20, 2)
    x[15:] += 2.
    g = knn(x, 5)
    g.set_gaussian(x, 1.)
    cliques = g.cliques()
    assert_true(len(np.unique(cliques)) > 1)


if __name__ == '__main__':
    import nose
    nose.run(argv=['', __file__])





from __future__ import absolute_import
from __future__ import print_function

# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np
import os
from nose.tools import assert_equal
from numpy.testing import (assert_array_almost_equal,
                           assert_array_equal,
                           assert_raises)
from ....core.image.image_spaces import (make_xyz_image,
                                         xyz_affine)
from ..parcel_analysis import (ParcelAnalysis, parcel_analysis,
                               _smooth_image_pair)


NSUBJ = 10
NLABELS = 10
SIZE = (50, 50, 50)
AFFINE = np.diag(np.concatenate((np.random.rand(3), np.ones((1,)))))


def test_smooth_image_pair():
    con_img = make_xyz_image(np.random.normal(0, 1, size=SIZE),
                             AFFINE, 'talairach')
    vcon_img = make_xyz_image(np.random.normal(0, 1, size=SIZE),
                             AFFINE, 'talairach')
    for sigma in (1, (1, 1.2, 0.8)):
        for method in ('default', 'spm'):
            scon_img, svcon_img = _smooth_image_pair(con_img, vcon_img,
                                                     sigma, method=method)
    assert_raises(ValueError, _smooth_image_pair, con_img, vcon_img, 1,
                  method='fsl')


def make_fake_data():
    con_imgs = [make_xyz_image(np.random.normal(0, 1, size=SIZE),
                               AFFINE, 'talairach') for i in range(NSUBJ)]
    parcel_img = make_xyz_image(np.random.randint(NLABELS, size=SIZE),
                                AFFINE, 'talairach')
    return con_imgs, parcel_img


def _test_parcel_analysis(smooth_method, parcel_info, vcon=False,
                          full_res=True):
    con_imgs, parcel_img = make_fake_data()
    if vcon:
        vcon_imgs = con_imgs
    else:
        vcon_imgs = None
    g = ParcelAnalysis(con_imgs, parcel_img,
                       vcon_imgs=vcon_imgs,
                       smooth_method=smooth_method,
                       parcel_info=parcel_info)
    t_map_img = g.t_map()
    assert_array_equal(t_map_img.shape, SIZE)
    assert_array_equal(xyz_affine(t_map_img), AFFINE)
    parcel_mu_img, parcel_prob_img = g.parcel_maps(full_res=full_res)
    assert_array_equal(parcel_mu_img.shape, SIZE)
    assert_array_equal(xyz_affine(parcel_mu_img), AFFINE)
    assert_array_equal(parcel_prob_img.shape, SIZE)
    assert_array_equal(xyz_affine(parcel_prob_img), AFFINE)
    assert parcel_prob_img.get_data().max() <= 1
    assert parcel_prob_img.get_data().min() >= 0
    outside = parcel_img.get_data() == 0
    assert_array_equal(t_map_img.get_data()[outside], 0)
    assert_array_equal(parcel_mu_img.get_data()[outside], 0)
    assert_array_equal(parcel_prob_img.get_data()[outside], 0)


def test_parcel_analysis():
    parcel_info = (list(range(NLABELS)), list(range(NLABELS)))
    _test_parcel_analysis('default', parcel_info)


def test_parcel_analysis_nonstandard():
    _test_parcel_analysis('default', None, vcon=True, full_res=False)


def test_parcel_analysis_spm():
    _test_parcel_analysis('spm', None)


def test_parcel_analysis_nosmooth():
    con_imgs, parcel_img = make_fake_data()
    msk_img = make_xyz_image(np.ones(SIZE, dtype='uint'),
                             AFFINE, 'talairach')
    X = np.random.normal(0, 1, size=(NSUBJ, 5))
    c = np.random.normal(0, 1, size=(5,))
    g = ParcelAnalysis(con_imgs, parcel_img,
                       msk_img=msk_img,
                       design_matrix=X,
                       cvect=c,
                       fwhm=0)
    t_map = g.t_map().get_data()
    m_error = np.abs(np.mean(t_map))
    v_error = np.abs(np.var(t_map) - (NSUBJ - 5) / float(NSUBJ - 7))
    print('Errors: %f (mean), %f (var)' % (m_error, v_error))
    assert m_error < .1
    assert v_error < .1


def _test_parcel_analysis_error(**kw):
    con_imgs, parcel_img = make_fake_data()
    return ParcelAnalysis(con_imgs, parcel_img, **kw)


def test_parcel_analysis_error():
    assert_raises(ValueError, _test_parcel_analysis_error,
                  vcon_imgs=list(range(NSUBJ + 1)))
    assert_raises(ValueError, _test_parcel_analysis_error,
                  cvect=np.ones(1))
    assert_raises(ValueError, _test_parcel_analysis_error,
                  design_matrix=np.random.rand(NSUBJ, 2))
    assert_raises(ValueError, _test_parcel_analysis_error,
                  design_matrix=np.random.rand(NSUBJ + 1, 2),
                  cvect=np.ones(2))
    assert_raises(ValueError, _test_parcel_analysis_error,
                  design_matrix=np.random.rand(NSUBJ, 2),
                  cvect=np.ones(3))


def test_parcel_analysis_write_mode():
    # find a subdirectory name that doesn't exist to check that
    # attempts to write in a non-existing directory do not raise
    # errors
    con_imgs, parcel_img = make_fake_data()
    subdirs = [o for o in os.listdir('.') if os.path.isdir(o)]
    res_path = 'a'
    while res_path in subdirs:
        res_path += 'a'
    p = ParcelAnalysis(con_imgs, parcel_img, res_path=res_path,
                       write_smoothed_images=True)
    assert_raises(IOError, p.dump_results)
    _ = p.t_map()
    _ = p.parcel_maps()


def test_parcel_analysis_function():
    con_imgs, parcel_img = make_fake_data()
    parcel_mu_img, parcel_prob_img = parcel_analysis(con_imgs, parcel_img)
    assert_array_equal(parcel_mu_img.shape, SIZE)
    assert_array_equal(xyz_affine(parcel_mu_img), AFFINE)
    assert_array_equal(parcel_prob_img.shape, SIZE)
    assert_array_equal(xyz_affine(parcel_prob_img), AFFINE)
    assert parcel_prob_img.get_data().max() <= 1
    assert parcel_prob_img.get_data().min() >= 0
    outside = parcel_img.get_data() == 0
    assert_array_equal(parcel_mu_img.get_data()[outside], 0)
    assert_array_equal(parcel_prob_img.get_data()[outside], 0)



""" Testing hrf module
"""
from __future__ import absolute_import

from os.path import dirname, join as pjoin

import numpy as np

from scipy.stats import gamma
import scipy.io as sio

from ..hrf import (
    gamma_params,
    gamma_expr,
    lambdify_t,
    spm_hrf_compat,
    spmt,
    dspmt,
    ddspmt,
    )

from nose.tools import assert_raises
from numpy.testing import assert_almost_equal


def test_gamma():
    t = np.linspace(0, 30, 5000)
    # make up some numbers
    pk_t = 5.0
    fwhm = 6.0
    # get the estimated parameters
    shape, scale, coef = gamma_params(pk_t, fwhm)
    # get distribution function
    g_exp = gamma_expr(pk_t, fwhm)
    # make matching standard distribution
    gf = gamma(shape, scale=scale).pdf
    # get values
    L1t = gf(t)
    L2t = lambdify_t(g_exp)(t)
    # they are the same bar a scaling factor
    nz = np.abs(L1t) > 1e-15
    sf = np.mean(L1t[nz] / L2t[nz])
    assert_almost_equal(L1t , L2t*sf)


def test_spm_hrf():
    # Regression tests for spm hrf, time derivative and dispersion derivative
    # Check that absolute values don't change (much) with different dt, and that
    # max values are roughly the same and in the same place in time
    for dt in 0.1, 0.01, 0.001:
        t_vec = np.arange(0, 32, dt)
        hrf = spmt(t_vec)
        assert_almost_equal(np.max(hrf), 0.21053, 5)
        assert_almost_equal(t_vec[np.argmax(hrf)], 5, 2)
        dhrf = dspmt(t_vec)
        assert_almost_equal(np.max(dhrf), 0.08, 3)
        assert_almost_equal(t_vec[np.argmax(dhrf)], 3.3, 1)
        dhrf = ddspmt(t_vec)
        assert_almost_equal(np.max(dhrf), 0.10, 2)
        assert_almost_equal(t_vec[np.argmax(dhrf)], 5.7, 1)
    # Test reversed time vector to check that order of time values does not
    # affect result
    rt_vec = np.arange(0, 32, 0.01)
    rhrf = spmt(rt_vec)
    assert_almost_equal(np.max(rhrf), 0.21053, 5)
    assert_almost_equal(t_vec[np.argmax(hrf)], 5, 2)


def test_spm_hrf_octave():
    # Test SPM hrf against output from SPM code running in Octave
    my_path = dirname(__file__)
    hrfs_path = pjoin(my_path, 'spm_hrfs.mat')
    # mat file resulting from make_hrfs.m
    hrfs_mat = sio.loadmat(hrfs_path, squeeze_me=True)
    params = hrfs_mat['params']
    hrfs = hrfs_mat['hrfs']
    for i, pvec in enumerate(params):
        dt, ppk, upk, pdsp, udsp, rat = pvec
        t_vec = np.arange(0, 32.1, dt)
        our_hrf = spm_hrf_compat(t_vec,
                                 peak_delay=ppk,
                                 peak_disp=pdsp,
                                 under_delay=upk,
                                 under_disp=udsp,
                                 p_u_ratio=rat)
        # Normalize integral to match SPM
        assert_almost_equal(our_hrf, hrfs[i])
    # Test basis functions
    # mat file resulting from get_td_dd.m
    bases_path = pjoin(my_path, 'spm_bases.mat')
    bases_mat = sio.loadmat(bases_path, squeeze_me=True)
    dt = bases_mat['dt']
    t_vec = np.arange(0, 32 + dt, dt)
    # SPM function divides by sum of values - revert with dt
    assert_almost_equal(spmt(t_vec), bases_mat['hrf'] / dt, 4)
    assert_almost_equal(dspmt(t_vec), bases_mat['dhrf'] / dt, 4)
    assert_almost_equal(ddspmt(t_vec), bases_mat['ddhrf'] / dt, 4)


def test_spm_hrf_errors():
    t_vec = np.arange(0, 32)
    # All 1s is fine
    res = spm_hrf_compat(t_vec, 1, 1, 1, 1)
    # 0 or negative raise error for other args
    args = [0]
    for i in range(4):
        assert_raises(ValueError, spm_hrf_compat, t_vec, *args)
        args[-1] = -1
        assert_raises(ValueError, spm_hrf_compat, t_vec, *args)
        args[-1] = 1
        args.append(0)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
''' Interfaces to SPM '''
from __future__ import with_statement
from __future__ import absolute_import

import os

import numpy as np

from scipy.io import savemat

from nibabel import load
from nibabel.tmpdirs import InTemporaryDirectory

from .matlab import run_matlab_script


class SpmInfo(object):
    def __init__(self):
        self._spm_path = None
        self._spm_ver = None

    def _set_properties(self):
        with InTemporaryDirectory():
            run_matlab_script(r"""
spm_path = spm('dir');
spm_ver = spm('ver');
fid = fopen('spm_stuff.txt', 'wt');
fprintf(fid, '%s\n', spm_path);
fprintf(fid, '%s\n', spm_ver);
fclose(fid);
""")
            with open('spm_stuff.txt', 'rt') as fobj:
                lines = fobj.readlines()
        self._spm_path = lines[0].strip()
        self._spm_ver = lines[1].strip()

    @property
    def spm_path(self):
        if self._spm_path is None:
            self._set_properties()
        return self._spm_path

    @property
    def spm_ver(self):
        if self._spm_ver is None:
            self._set_properties()
        return self._spm_ver


spm_info = SpmInfo()


def make_job(jobtype, jobname, contents):
    return {'jobs':[{jobtype:[{jobname:contents}]}]}


# XXX this should be moved into a matdict class or something
def fltcols(vals):
    ''' Trivial little function to make 1xN float vector '''
    return np.atleast_2d(np.array(vals, dtype=float))


def run_jobdef(jobdef):
    script = """
load pyjobs;
spm_jobman('run', jobs);
"""
    # Need initcfg for SPM8
    if spm_info.spm_ver != 'SPM5':
        script = "spm_jobman('initcfg');\n" + script
    with InTemporaryDirectory():
        savemat('pyjobs.mat', jobdef, oned_as='row')
        run_matlab_script(script)


def scans_for_fname(fname):
    img = load(fname)
    n_scans = img.shape[3]
    scans = np.zeros((n_scans, 1), dtype=object)
    for sno in range(n_scans):
        scans[sno] = '%s,%d' % (fname, sno+1)
    return scans


def scans_for_fnames(fnames):
    n_sess = len(fnames)
    sess_scans = np.zeros((1,n_sess), dtype=object)
    for sess in range(n_sess):
        sess_scans[0,sess] = scans_for_fname(fnames[sess])
    return sess_scans


def fname_presuffix(fname, prefix='', suffix='', use_ext=True):
    pth, fname = os.path.split(fname)
    fname, ext = os.path.splitext(fname)
    if not use_ext:
        ext = ''
    return os.path.join(pth, prefix+fname+suffix+ext)


def fnames_presuffix(fnames, prefix='', suffix=''):
    f2 = []
    for fname in fnames:
        f2.append(fname_presuffix(fname, prefix, suffix))
    return f2





from __future__ import absolute_import
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from .glm import models, contrast, ols, load

from warnings import warn

warn('Module nipy.labs.glm deprecated, will be removed. '
     'Please use nipy.modalities.fmri.glm instead.',
     FutureWarning,
     stacklevel=2)

from nipy.testing import Tester
test = Tester().test
bench = Tester().bench



""" Parcel-based group analysis of multi-subject image data.

Routines implementing Bayesian inference on group-level effects
assumed to be constant within given brain parcels. The model accounts
for both estimation errors and localization uncertainty in reference
space of first-level images.

See:

Keller, Merlin et al (2008). Dealing with Spatial Normalization Errors
in fMRI Group Inference using Hierarchical Modeling. *Statistica
Sinica*; 18(4).

Keller, Merlin et al (2009). Anatomically Informed Bayesian Model
Selection for fMRI Group Data Analysis. *In MICCAI'09, Lecture Notes
in Computer Science*; 5762:450--457.

Roche, Alexis (2012). OHBM'12 talk, slides at:
https://sites.google.com/site/alexisroche/slides/Talk_Beijing12.pdf
"""
from __future__ import absolute_import
from os.path import join
import warnings
import numpy as np
import scipy.ndimage as nd
import scipy.stats as ss
from ..statistics.bayesian_mixed_effects import two_level_glm
from ..statistics.histogram import histogram
from ..registration import resample
from ..kernel_smooth import fwhm2sigma
from ...fixes.nibabel import io_orientation
from ...core.image.image_spaces import (make_xyz_image,
                                        xyz_affine)
from ... import save_image

SIGMA_MIN = 1e-5
NDIM = 3  # This will work for 3D images


def _gaussian_filter(x, msk, sigma):
    """
    Smooth a multidimensional array `x` using a Gaussian filter with
    axis-wise standard deviations given by `sigma`, after padding `x`
    with zeros within a mask `msk`.
    """
    x[msk] = 0.
    gx = nd.gaussian_filter(x, sigma)
    norma = 1 - nd.gaussian_filter(msk.astype(float), sigma)
    gx[True - msk] /= norma[True - msk]
    gx[msk] = 0.
    return gx


def _gaussian_energy_1d(sigma):
    """
    Compute the integral of a one-dimensional squared three-dimensional
    Gaussian kernel with axis-wise standard deviation `sigma`.
    """
    mask_half_size = np.ceil(5 * sigma).astype(int)
    mask_size = 2 * mask_half_size + 1
    x = np.zeros(mask_size)
    x[mask_half_size] = 1
    y = nd.gaussian_filter1d(x, sigma)
    K = np.sum(y ** 2) / np.sum(y)
    return K


def _gaussian_energy(sigma):
    """
    Compute the integral of a squared three-dimensional Gaussian
    kernel with axis-wise standard deviations `sigma`.
    """
    sigma = np.asarray(sigma)
    if sigma.size == 1:
        sigma = np.repeat(sigma, NDIM)
    # Use kernel separability to save memory
    return np.prod([_gaussian_energy_1d(s) for s in sigma])


def _smooth(con, vcon, msk, sigma):
    """
    Integrate spatial uncertainty in standard space assuming that
    localization errors follow a zero-mean Gaussian distribution with
    axis-wise standard deviations `sigma` in voxel units. The expected
    Euclidean norm of registration errors is sqrt(NDIM) * sigma.
    """
    scon = _gaussian_filter(con, msk, sigma)
    svcon = _gaussian_filter(con ** 2, msk, sigma) - scon ** 2
    if vcon is not None:
        svcon += _gaussian_filter(vcon, msk, sigma)
    return scon, svcon


def _smooth_spm(con, vcon, msk, sigma):
    """
    Given a contrast image `con` and the corresponding variance image
    `vcon`, both assumed to be estimated from non-smoothed first-level
    data, compute what `con` and `vcon` would have been had the data
    been smoothed with a Gaussian kernel.
    """
    scon = _gaussian_filter(con, msk, sigma)
    K = _gaussian_energy(sigma)
    if vcon is not None:
        svcon = K * _gaussian_filter(vcon, msk, sigma / np.sqrt(2))
    else:
        svcon = np.zeros(con.shape)
    return scon, svcon


def _smooth_image_pair(con_img, vcon_img, sigma, method='default'):
    """
    Smooth an input image and associated variance image using either
    the spatial uncertainty accounting method consistent with Keller
    et al's model, or the SPM approach.
    """
    if method == 'default':
        smooth_fn = _smooth
    elif method == 'spm':
        smooth_fn = _smooth_spm
    else:
        raise ValueError('Unknown smoothing method')
    con = con_img.get_data()
    if vcon_img is not None:
        vcon = con_img.get_data()
    else:
        vcon = None
    msk = np.isnan(con)
    scon, svcon = smooth_fn(con, vcon, msk, sigma)
    scon_img = make_xyz_image(scon, xyz_affine(con_img),
                              con_img.reference)
    svcon_img = make_xyz_image(svcon, xyz_affine(con_img),
                               con_img.reference)
    return scon_img, svcon_img


def _save_image(img, path):
    try:
        save_image(img, path)
    except:
        warnings.warn('Could not write image: %s' % path, UserWarning)


class ParcelAnalysis(object):

    def __init__(self, con_imgs, parcel_img, parcel_info=None,
                 msk_img=None, vcon_imgs=None,
                 design_matrix=None, cvect=None,
                 fwhm=8, smooth_method='default',
                 res_path=None, write_smoothed_images=False):
        """
        Bayesian parcel-based analysis.

        Given a sequence of independent images registered to a common
        space (for instance, a set of contrast images from a
        first-level fMRI analysis), perform a second-level analysis
        assuming constant effects throughout parcels defined from a
        given label image in reference space. Specifically, a model of
        the following form is assumed:

        Y = X * beta + variability,

        where Y denotes the input image sequence, X is a design
        matrix, and beta are parcel-wise parameter vectors. The
        algorithm computes the Bayesian posterior probability of beta
        in each parcel using an expectation propagation scheme.

        Parameters
        ----------
        con_imgs: sequence of nipy-like images
          Images input to the group analysis.
        parcel_img: nipy-like image
          Label image where each label codes for a parcel.
        parcel_info: sequence of arrays, optional
          A sequence of two arrays with same length equal to the
          number of distinct parcels consistently with the
          `parcel_img` argument. The first array gives parcel names
          and the second, parcel values, i.e., corresponding
          intensities in the associated parcel image. By default,
          parcel values are taken as
          `np.unique(parcel_img.get_data())` and parcel names are
          these values converted to strings.
        msk_img: nipy-like image, optional
          Binary mask to restrict analysis. By default, analysis is
          carried out on all parcels with nonzero value.
        vcon_imgs: sequece of nipy-like images, optional
          First-level variance estimates corresponding to
          `con_imgs`. This is useful if the input images are
          "noisy". By default, first-level variances are assumed to be
          zero.
        design_matrix: array, optional
          If None, a one-sample analysis model is used. Otherwise, an
          array with shape (n, p) where `n` matches the number of
          input scans, and `p` is the number of regressors.
        cvect: array, optional
          Contrast vector of interest. The method makes an inference
          on the contrast defined as the dot product cvect'*beta,
          where beta are the unknown parcel-wise effects. If None,
          `cvect` is assumed to be np.array((1,)). However, the
          `cvect` argument is mandatory if `design_matrix` is
          provided.
        fwhm: float, optional
          A parameter that represents the localization uncertainty in
          reference space in terms of the full width at half maximum
          of an isotropic Gaussian kernel.
        smooth_method: str, optional
          One of 'default' and 'spm'. Setting `smooth_method=spm`
          results in simply smoothing the input images using a
          Gaussian kernel, while the default method involves more
          complex smoothing in order to propagate spatial uncertainty
          into the inference process.
        res_path: str, optional
          An existing path to write output images. If None, no output
          is written.
        write_smoothed_images: bool, optional
          Specify whether smoothed images computed throughout the
          inference process are to be written on disk in `res_path`.
        """
        self.smooth_method = smooth_method
        self.con_imgs = con_imgs
        self.vcon_imgs = vcon_imgs
        self.n_subjects = len(con_imgs)
        if self.vcon_imgs is not None:
            if not self.n_subjects == len(vcon_imgs):
                raise ValueError('List of contrasts and variances'
                                 ' do not have the same length')
        if msk_img is None:
            self.msk = None
        else:
            self.msk = msk_img.get_data().astype(bool).squeeze()
        self.res_path = res_path

        # design matrix
        if design_matrix is None:
            self.design_matrix = np.ones(self.n_subjects)
            self.cvect = np.ones((1,))
            if cvect is not None:
                raise ValueError('No contrast vector expected')
        else:
            self.design_matrix = np.asarray(design_matrix)
            if cvect is None:
                raise ValueError('`cvect` cannot be None with'
                                 ' provided design matrix')
            self.cvect = np.asarray(cvect)
            if not self.design_matrix.shape[0] == self.n_subjects:
                raise ValueError('Design matrix shape is inconsistent'
                                 ' with number of input images')
            if not len(self.cvect) == self.design_matrix.shape[1]:
                raise ValueError('Design matrix shape is inconsistent'
                                 ' with provided `cvect`')

        # load the parcellation and resample it at the appropriate
        # resolution
        self.reference = parcel_img.reference
        self.parcel_full_res = parcel_img.get_data().astype('uint').squeeze()
        self.affine_full_res = xyz_affine(parcel_img)
        parcel_img = make_xyz_image(self.parcel_full_res,
                                    self.affine_full_res,
                                    self.reference)
        self.affine = xyz_affine(self.con_imgs[0])
        parcel_img_rsp = resample(parcel_img,
                                  reference=(self.con_imgs[0].shape,
                                             self.affine),
                                  interp_order=0)
        self.parcel = parcel_img_rsp.get_data().astype('uint').squeeze()
        if self.msk is None:
            self.msk = self.parcel > 0

        # get parcel labels and values
        if parcel_info is None:
            self._parcel_values = np.unique(self.parcel)
            self._parcel_labels = self._parcel_values.astype(str)
        else:
            self._parcel_labels = np.asarray(parcel_info[0]).astype(str)
            self._parcel_values = np.asarray(parcel_info[1])

        # determine smoothing kernel size, which involves converting
        # the input full-width-at-half-maximum parameter given in mm
        # to standard deviation in voxel units.
        orient = io_orientation(self.affine)[:, 0].astype(int)
        # `orient` is an array, so this slicing leads to advanced indexing.
        voxsize = np.abs(self.affine[orient, list(range(3))])
        self.sigma = np.maximum(fwhm2sigma(fwhm) / voxsize, SIGMA_MIN)

        # run approximate belief propagation
        self._smooth_images(write_smoothed_images)
        self._voxel_level_inference()
        self._parcel_level_inference()

    def _smooth_images(self, write):
        """
        Smooth input contrast images to account for localization
        uncertainty in reference space.
        """
        cons, vcons = [], []
        for i in range(self.n_subjects):
            con = self.con_imgs[i]
            if self.vcon_imgs is not None:
                vcon = self.vcon_imgs[i]
            else:
                vcon = None
            scon, svcon = _smooth_image_pair(con, vcon, self.sigma,
                                             method=self.smooth_method)
            if write and self.res_path is not None:
                _save_image(scon, join(self.res_path,
                                       'scon' + str(i) + '.nii.gz'))
                _save_image(svcon, join(self.res_path,
                                        'svcon' + str(i) + '.nii.gz'))
            cons += [scon.get_data()[self.msk]]
            vcons += [svcon.get_data()[self.msk]]

        self.cons = np.array(cons)
        self.vcons = np.array(vcons)

    def _voxel_level_inference(self, mfx=True):
        """
        Estimate voxel-level group parameters using mixed effects
        variational Bayes algorithm.
        """
        beta, s2, dof = two_level_glm(self.cons, self.vcons,
                                      self.design_matrix)

        self.beta = np.dot(self.cvect, beta)
        if self.design_matrix.ndim == 1:
            self.vbeta = s2 * (self.cvect[0] ** 2\
                                   / np.sum(self.design_matrix ** 2))
        else:
            tmp = np.linalg.inv(np.dot(self.design_matrix.T,
                                       self.design_matrix))
            self.vbeta = s2 * np.dot(self.cvect.T, np.dot(tmp, self.cvect))
        self.dof = dof

    def _parcel_level_inference(self):
        """
        Estimate parcel-level group parameters using mixed effects
        variational Bayes algorithm.
        """
        parcel_masked = self.parcel[self.msk]
        values = np.where(histogram(parcel_masked) > 0)[0][1:]

        prob = np.zeros(len(values))
        mu = np.zeros(len(values))
        s2 = np.zeros(len(values))
        dof = np.zeros(len(values))
        labels = []

        # For each parcel, estimate parcel-level parameters using a
        # mxf model
        for i in range(len(values)):
            mask = parcel_masked == values[i]
            y = self.beta[mask]
            vy = self.vbeta[mask]
            npts = y.size
            try:
                mu[i], s2[i], dof[i] = two_level_glm(y, vy, np.ones(npts))
                prob[i] = ss.t.cdf(float(mu[i] / np.sqrt(s2[i] / npts)),
                                   dof[i])
            except:
                prob[i] = 0
            idx = int(np.where(self._parcel_values == values[i])[0])
            labels += [self._parcel_labels[idx]]

        # Sort labels by ascending order of mean values
        I = np.argsort(-mu)
        self.parcel_values = values[I]
        self.parcel_labels = np.array(labels)[I]
        self.parcel_prob = prob[I]
        self.parcel_mu = mu[I]
        self.parcel_s2 = s2[I]
        self.parcel_dof = dof[I]

    def dump_results(self, path=None):
        """
        Save parcel analysis information in NPZ file.
        """
        if path is None and self.res_path is not None:
            path = self.res_path
        else:
            path = '.'
        np.savez(join(path, 'parcel_analysis.npz'),
                 values=self.parcel_values,
                 labels=self.parcel_labels,
                 prob=self.parcel_prob,
                 mu=self.parcel_mu,
                 s2=self.parcel_s2,
                 dof=self.parcel_dof)

    def t_map(self):
        """
        Compute voxel-wise t-statistic map. This map is different from
        what you would get from an SPM-style mass univariate analysis
        because the method accounts for both spatial uncertainty in
        reference space and possibly errors on first-level inputs (if
        variance images are provided).

        Returns
        -------
        tmap_img: nipy image
          t-statistic map.
        """
        tmap = np.zeros(self.msk.shape)
        beta = self.beta
        var = self.vbeta
        tmap[self.msk] = beta / np.sqrt(var)
        tmap_img = make_xyz_image(tmap, self.affine, self.reference)
        if self.res_path is not None:
            _save_image(tmap_img, join(self.res_path, 'tmap.nii.gz'))
            tmp = np.zeros(self.msk.shape)
            tmp[self.msk] = beta
            _save_image(make_xyz_image(tmp, self.affine, self.reference),
                        join(self.res_path, 'beta.nii.gz'))
            tmp[self.msk] = var
            _save_image(make_xyz_image(tmp, self.affine, self.reference),
                        join(self.res_path, 'vbeta.nii.gz'))
        return tmap_img

    def parcel_maps(self, full_res=True):
        """
        Compute parcel-based posterior contrast means and positive
        contrast probabilities.

        Parameters
        ----------
        full_res: boolean
         If True, the output images will be at the same resolution as
         the parcel image. Otherwise, resolution will match the
         first-level images.

        Returns
        -------
        pmap_mu_img: nipy image
          Image of posterior contrast means for each parcel.
        pmap_prob_img: nipy image
          Corresponding image of posterior probabilities of positive
          contrast.
        """
        if full_res:
            parcel = self.parcel_full_res
            affine = self.affine_full_res
        else:
            parcel = self.parcel
            affine = self.affine
        pmap_prob = np.zeros(parcel.shape)
        pmap_mu = np.zeros(parcel.shape)
        for label, prob, mu in zip(self.parcel_values,
                                    self.parcel_prob,
                                    self.parcel_mu):
            pmap_prob[parcel == label] = prob
            pmap_mu[parcel == label] = mu

        pmap_prob_img = make_xyz_image(pmap_prob, affine, self.reference)
        pmap_mu_img = make_xyz_image(pmap_mu, affine, self.reference)

        if self.res_path is not None:
            _save_image(pmap_prob_img,
                        join(self.res_path, 'parcel_prob.nii.gz'))
            _save_image(pmap_mu_img,
                        join(self.res_path, 'parcel_mu.nii.gz'))

        return pmap_mu_img, pmap_prob_img


def parcel_analysis(con_imgs, parcel_img,
                    msk_img=None, vcon_imgs=None,
                    design_matrix=None, cvect=None,
                    fwhm=8, smooth_method='default',
                    res_path=None):
    """
    Helper function for Bayesian parcel-based analysis.

    Given a sequence of independent images registered to a common
    space (for instance, a set of contrast images from a first-level
    fMRI analysis), perform a second-level analysis assuming constant
    effects throughout parcels defined from a given label image in
    reference space. Specifically, a model of the following form is
    assumed:

    Y = X * beta + variability,

    where Y denotes the input image sequence, X is a design matrix,
    and beta are parcel-wise parameter vectors. The algorithm computes
    the Bayesian posterior probability of cvect'*beta, where cvect is
    a given contrast vector, in each parcel using an expectation
    propagation scheme.

    Parameters
    ----------
    con_imgs: sequence of nipy-like images
      Images input to the group analysis.
    parcel_img: nipy-like image
      Label image where each label codes for a parcel.
    msk_img: nipy-like image, optional
      Binary mask to restrict analysis. By default, analysis is
      carried out on all parcels with nonzero value.
    vcon_imgs: sequece of nipy-like images, optional
      First-level variance estimates corresponding to `con_imgs`. This
      is useful if the input images are "noisy". By default,
      first-level variances are assumed to be zero.
    design_matrix: array, optional
      If None, a one-sample analysis model is used. Otherwise, an
      array with shape (n, p) where `n` matches the number of input
      scans, and `p` is the number of regressors.
    cvect: array, optional
      Contrast vector of interest. The method makes an inference on
      the contrast defined as the dot product cvect'*beta, where beta
      are the unknown parcel-wise effects. If None, `cvect` is assumed
      to be np.array((1,)). However, the `cvect` argument is mandatory
      if `design_matrix` is provided.
    fwhm: float, optional
      A parameter that represents the localization uncertainty in
      reference space in terms of the full width at half maximum of an
      isotropic Gaussian kernel.
    smooth_method: str, optional
      One of 'default' and 'spm'. Setting `smooth_method=spm` results
      in simply smoothing the input images using a Gaussian kernel,
      while the default method involves more complex smoothing in
      order to propagate spatial uncertainty into the inference
      process.
    res_path: str, optional
      An existing path to write output images. If None, no output is
      written.

    Returns
    -------
    pmap_mu_img: nipy image
      Image of posterior contrast means for each parcel.
    pmap_prob_img: nipy image
      Corresponding image of posterior probabilities of positive
      contrast.
    """
    p = ParcelAnalysis(con_imgs, parcel_img, parcel_info=None,
                       msk_img=msk_img, vcon_imgs=vcon_imgs,
                       design_matrix=design_matrix, cvect=cvect,
                       fwhm=fwhm, smooth_method=smooth_method,
                       res_path=res_path)
    return p.parcel_maps()



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Convenience functions for specifying a design in the GLM
"""
from __future__ import absolute_import

import itertools

import numpy as np

from nipy.algorithms.statistics.utils import combinations
from nipy.algorithms.statistics.formula import formulae
from nipy.algorithms.statistics.formula.formulae import (
    Formula, Factor, Term, make_recarray)

from .utils import (events, blocks, fourier_basis as fourier_basis_sym,
                    convolve_functions, T)

from .hrf import glover

def fourier_basis(t, freq):
    """
    Create a design matrix with columns given by the Fourier
    basis with a given set of frequencies.

    Parameters
    ----------
    t : np.ndarray
        An array of np.float values at which to evaluate
        the design. Common examples would be the acquisition
        times of an fMRI image.
    freq : sequence of float
        Frequencies for the terms in the Fourier basis.

    Returns
    -------
    X : np.ndarray

    Examples
    --------
    >>> t = np.linspace(0,50,101)
    >>> drift = fourier_basis(t, np.array([4,6,8]))
    >>> drift.shape
    (101, 6)
    """
    tval = make_recarray(t, ['t'])
    f = fourier_basis_sym(freq)
    return f.design(tval, return_float=True)


def natural_spline(tvals, knots=None, order=3, intercept=True):
    """ Design matrix with columns given by a natural spline order `order`

    Return design matrix with natural splines with knots `knots`, order
    `order`.  If `intercept` == True (the default), add constant column.

    Parameters
    ----------
    tvals : np.array
        Time values
    knots : None or sequence, optional
       Sequence of float.  Default None (same as empty list)
    order : int, optional
       Order of the spline. Defaults to a cubic (==3)
    intercept : bool, optional
       If True, include a constant function in the natural
       spline. Default is False

    Returns
    -------
    X : np.ndarray

    Examples
    --------
    >>> tvals = np.linspace(0,50,101)
    >>> drift = natural_spline(tvals, knots=[10,20,30,40])
    >>> drift.shape
    (101, 8)
    """
    tvals = make_recarray(tvals, ['t'])
    t = Term('t')
    f = formulae.natural_spline(t, knots=knots, order=order, intercept=intercept)
    return f.design(tvals, return_float=True)


def _build_formula_contrasts(spec, fields, order):
    """ Build formula and contrast in event / block space

    Parameters
    ----------
    spec : stuctured array
        Structured array containing at least fields listed in `fields`.
    fields : sequence of str
        Sequence of field names containing names of factors.
    order : int
        Maximum order of interactions between main effects.

    Returns
    -------
    e_factors : :class:`Formula` instance
        Formula for factors given by `fields`
    e_contrasts : dict
        Dictionary containing contrasts of main effects and interactions
        between factors.
    """
    e_factors = [Factor(n, np.unique(spec[n])) for n in fields]
    e_formula = np.product(e_factors)
    e_contrasts = {}
    # Add contrasts for factors and factor interactions
    max_order = min(len(e_factors), order)
    for i in range(1, max_order + 1):
        for comb in combinations(zip(fields, e_factors), i):
            names = [c[0] for c in comb]
            # Collect factors where there is more than one level
            fs = [fc.main_effect for fn, fc in comb if len(fc.levels) > 1]
            if len(fs) > 0:
                e_contrast = np.product(fs).design(spec)
                e_contrasts[":".join(names)] = e_contrast
    e_contrasts['constant'] = formulae.I.design(spec)
    return e_formula, e_contrasts


def event_design(event_spec, t, order=2, hrfs=(glover,),
                 level_contrasts=False):
    """ Create design matrix at times `t` for event specification `event_spec`

    Create a design matrix for linear model based on an event specification
    `event_spec`, evaluating the design rows at a sequence of time values `t`.
    Each column in the design matrix will be convolved with each HRF in `hrfs`.

    Parameters
    ----------
    event_spec : np.recarray
       A recarray having at least a field named 'time' signifying the event
       time, and all other fields will be treated as factors in an ANOVA-type
       model.  If there is no field other than time, add a single-level
       placeholder event type ``_event_``.
    t : np.ndarray
       An array of np.float values at which to evaluate the design. Common
       examples would be the acquisition times of an fMRI image.
    order : int, optional
       The highest order interaction to be considered in constructing
       the contrast matrices.
    hrfs : sequence, optional
       A sequence of (symbolic) HRFs that will be convolved with each event.
       Default is ``(glover,)``.
    level_contrasts : bool, optional
       If True, generate contrasts for each individual level of each factor.

    Returns
    -------
    X : np.ndarray
       The design matrix with ``X.shape[0] == t.shape[0]``. The number of
       columns will depend on the other fields of `event_spec`.
    contrasts : dict
       Dictionary of contrasts that is expected to be of interest from the
       event specification. Each interaction / effect up to a given order will
       be returned. Also, a contrast is generated for each interaction / effect
       for each HRF specified in `hrfs`.
    """
    fields = list(event_spec.dtype.names)
    if 'time' not in fields:
        raise ValueError('expecting a field called "time"')
    fields.pop(fields.index('time'))
    if len(fields) == 0:  # No factors specified, make generic event
        event_spec = make_recarray(zip(event_spec['time'],
                                       itertools.cycle([1])),
                                   ('time', '_event_'))
        fields = ['_event_']
    e_formula, e_contrasts = _build_formula_contrasts(
        event_spec, fields, order)
    # Design and contrasts in block space
    # TODO: make it so I don't have to call design twice here
    # to get both the contrasts and the e_X matrix as a recarray
    e_X = e_formula.design(event_spec)
    e_dtype = e_formula.dtype

    # Now construct the design in time space
    t_terms = []
    t_contrasts = {}
    for l, h in enumerate(hrfs):
        for n in e_dtype.names:
            term = events(event_spec['time'], amplitudes=e_X[n], f=h)
            t_terms  += [term]
            if level_contrasts:
                t_contrasts['%s_%d' % (n, l)] = Formula([term])
        for n, c in e_contrasts.items():
            t_contrasts["%s_%d" % (n, l)] = Formula([ \
                 events(event_spec['time'], amplitudes=c[nn], f=h)
                 for i, nn in enumerate(c.dtype.names)])
    t_formula = Formula(t_terms)

    tval = make_recarray(t, ['t'])
    X_t, c_t = t_formula.design(tval, contrasts=t_contrasts)
    return X_t, c_t


def block_design(block_spec, t, order=2, hrfs=(glover,),
                 convolution_padding=5.,
                 convolution_dt=0.02,
                 hrf_interval=(0.,30.),
                 level_contrasts=False):
    """ Create design matrix at times `t` for blocks specification `block_spec`

    Create design matrix for linear model from a block specification
    `block_spec`,  evaluating design rows at a sequence of time values `t`.
    Each column in the design matrix will be convolved with each HRF in `hrfs`.

    Parameters
    ----------
    block_spec : np.recarray
       A recarray having at least a field named 'start' and a field named 'end'
       signifying the block onset and offset times. All other fields will be
       treated as factors in an ANOVA-type model.  If there is no field other
       than 'start' and 'end', add a single-level placeholder block type
       ``_block_``.
    t : np.ndarray
       An array of np.float values at which to evaluate the design. Common
       examples would be the acquisition times of an fMRI image.
    order : int, optional
       The highest order interaction to be considered in constructing the
       contrast matrices.
    hrfs : sequence, optional
       A sequence of (symbolic) HRFs that will be convolved with each block.
       Default is ``(glover,)``.
    convolution_padding : float, optional
       A padding for the convolution with the HRF. The intervals
       used for the convolution are the smallest 'start' minus this
       padding to the largest 'end' plus this padding.
    convolution_dt : float, optional
       Time step for high-resolution time course for use in convolving the
       blocks with each HRF.
    hrf_interval: length 2 sequence of floats, optional
       Interval over which the HRF is assumed supported, used in the
       convolution.
    level_contrasts : bool, optional
       If true, generate contrasts for each individual level
       of each factor.

    Returns
    -------
    X : np.ndarray
       The design matrix with ``X.shape[0] == t.shape[0]``. The number of
       columns will depend on the other fields of `block_spec`.
    contrasts : dict
       Dictionary of contrasts that are expected to be of interest from the
       block specification. Each interaction / effect up to a given order will
       be returned. Also, a contrast is generated for each interaction / effect
       for each HRF specified in `hrfs`.
    """
    fields = list(block_spec.dtype.names)
    if 'start' not in fields or 'end' not in fields:
        raise ValueError('expecting fields called "start" and "end"')
    fields.pop(fields.index('start'))
    fields.pop(fields.index('end'))
    if len(fields) == 0:  # No factors specified, make generic block
        block_spec = make_recarray(zip(block_spec['start'],
                                       block_spec['end'],
                                       itertools.cycle([1])),
                                   ('start', 'end', '_block_'))
        fields = ['_block_']
    e_formula, e_contrasts = _build_formula_contrasts(
        block_spec, fields, order)
    # Design and contrasts in block space
    # TODO: make it so I don't have to call design twice here
    # to get both the contrasts and the e_X matrix as a recarray
    e_X = e_formula.design(block_spec)
    e_dtype = e_formula.dtype

    # Now construct the design in time space
    block_times = np.array([(s,e) for s, e in zip(block_spec['start'], 
                                                  block_spec['end'])])
    convolution_interval = (block_times.min() - convolution_padding, 
                            block_times.max() + convolution_padding)

    t_terms = []
    t_contrasts = {}
    for l, h in enumerate(hrfs):
        for n in e_dtype.names:
            B = blocks(block_times, amplitudes=e_X[n])
            term = convolve_functions(B, h(T), 
                                      convolution_interval,
                                      hrf_interval,
                                      convolution_dt)
            t_terms += [term]
            if level_contrasts:
                t_contrasts['%s_%d' % (n, l)] = Formula([term])
        for n, c in e_contrasts.items():
            F = []
            for i, nn in enumerate(c.dtype.names):
                B = blocks(block_times, amplitudes=c[nn])
                F.append(convolve_functions(B, h(T),
                                            convolution_interval,
                                            hrf_interval,
                                            convolution_dt))
            t_contrasts["%s_%d" % (n, l)] = Formula(F)
    t_formula = Formula(t_terms)

    tval = make_recarray(t, ['t'])
    X_t, c_t = t_formula.design(tval, contrasts=t_contrasts)
    return X_t, c_t


def stack2designs(old_X, new_X, old_contrasts={}, new_contrasts={}):
    """
    Add some columns to a design matrix that has contrasts matrices
    already specified, adding some possibly new contrasts as well.

    This basically performs an np.hstack of old_X, new_X
    and makes sure the contrast matrices are dealt with accordingly.

    If two contrasts have the same name, an exception is raised.

    Parameters
    ----------
    old_X : np.ndarray
       A design matrix
    new_X : np.ndarray
       A second design matrix to be stacked with old_X
    old_contrast : dict
       Dictionary of contrasts in the old_X column space
    new_contrasts : dict
       Dictionary of contrasts in the new_X column space

    Returns
    -------
    X : np.ndarray
       A new design matrix:  np.hstack([old_X, new_X])
    contrasts : dict
       The new contrast matrices reflecting changes to the columns.
    """
    contrasts = {}
    old_X = np.asarray(old_X)
    new_X = np.asarray(new_X)
    if old_X.size == 0:
        return new_X, new_contrasts
    if new_X.size == 0:
        return old_X, old_contrasts

    if old_X.ndim == 1:
        old_X = old_X[:, None]
    if new_X.ndim == 1:
        new_X = new_X[:, None]

    X = np.hstack([old_X, new_X])

    if set(old_contrasts.keys()).intersection(new_contrasts.keys()) != set([]):
        raise ValueError('old and new contrasts must have different names')

    for n, c in old_contrasts.items():
        if c.ndim > 1:
            cm = np.zeros((c.shape[0], X.shape[1]))
            cm[:,:old_X.shape[1]] = c
        else:
            cm = np.zeros(X.shape[1])
            cm[:old_X.shape[1]] = c
        contrasts[n] = cm

    for n, c in new_contrasts.items():
        if c.ndim > 1:
            cm = np.zeros((c.shape[0], X.shape[1]))
            cm[:,old_X.shape[1]:] = c
        else:
            cm = np.zeros(X.shape[1])
            cm[old_X.shape[1]:] = c
        contrasts[n] = cm

    return X, contrasts


def stack_contrasts(contrasts, name, keys):
    """
    Create a new F-contrast matrix called 'name'
    based on a sequence of keys. The contrast
    is added to contrasts, in-place.

    Parameters
    ----------
    contrasts : dict
       Dictionary of contrast matrices
    name : str
       Name of new contrast. Should not already be a key of contrasts.
    keys : sequence of str
       Keys of contrasts that are to be stacked.

    Returns
    -------
    None
    """
    if name in contrasts.keys():
        raise ValueError('contrast "%s" already exists' % name)

    contrasts[name] = np.vstack([contrasts[k] for k in keys])


def stack_designs(*pairs):
    """ Stack a sequence of design / contrast dictionary pairs

    Uses multiple calls to :func:`stack2designs`

    Parameters
    ----------
    \*pairs : sequence
        Elements of either (np.ndarray, dict) or (np.ndarray,) or np.ndarray

    Returns
    -------
    X : np.ndarray
       new design matrix:  np.hstack([old_X, new_X])
    contrasts : dict
       The new contrast matrices reflecting changes to the columns.
    """
    X = []
    contrasts = {}
    for p in pairs:
        if isinstance(p, np.ndarray):
            new_X = p; new_con = {}
        elif len(p) == 1:  # Length one sequence
            new_X = p[0]; new_con = {}
        else:  # Length 2 sequence
            new_X, new_con = p
        X, contrasts = stack2designs(X, new_X, contrasts, new_con)
    return X, contrasts


def openfmri2nipy(ons_dur_amp):
    """ Contents of OpenFMRI condition file `ons_dur_map` as nipy recarray

    Parameters
    ----------
    ons_dur_amp : str or array
        Path to OpenFMRI stimulus file or 2D array containing three columns
        corresponding to onset, duration, amplitude.

    Returns
    -------
    block_spec : array
        Structured array with fields "start" (corresponding to onset time),
        "end" (onset time plus duration), "amplitude".
    """
    if not isinstance(ons_dur_amp, np.ndarray):
        ons_dur_amp = np.loadtxt(ons_dur_amp)
    onsets, durations, amplitudes = ons_dur_amp.T
    return make_recarray(
        np.column_stack((onsets, onsets + durations, amplitudes)),
        names=['start', 'end', 'amplitude'],
        drop_name_dim=True)


def block_amplitudes(name, block_spec, t, hrfs=(glover,),
                     convolution_padding=5.,
                     convolution_dt=0.02,
                     hrf_interval=(0.,30.)):
    """ Design matrix at times `t` for blocks specification `block_spec`

    Create design matrix for linear model from a block specification
    `block_spec`,  evaluating design rows at a sequence of time values `t`.

    `block_spec` may specify amplitude of response for each event, if different
    (see description of `block_spec` parameter below).

    The on-off step function implied by `block_spec` will be convolved with
    each HRF in `hrfs` to form a design matrix shape ``(len(t), len(hrfs))``.

    Parameters
    ----------
    name : str
        Name of condition
    block_spec : np.recarray or array-like
       A recarray having fields ``start, end, amplitude``, or a 2D ndarray /
       array-like with three columns corresponding to start, end, amplitude.
    t : np.ndarray
       An array of np.float values at which to evaluate the design. Common
       examples would be the acquisition times of an fMRI image.
    hrfs : sequence, optional
       A sequence of (symbolic) HRFs that will be convolved with each block.
       Default is ``(glover,)``.
    convolution_padding : float, optional
       A padding for the convolution with the HRF. The intervals
       used for the convolution are the smallest 'start' minus this
       padding to the largest 'end' plus this padding.
    convolution_dt : float, optional
       Time step for high-resolution time course for use in convolving the
       blocks with each HRF.
    hrf_interval: length 2 sequence of floats, optional
       Interval over which the HRF is assumed supported, used in the
       convolution.

    Returns
    -------
    X : np.ndarray
       The design matrix with ``X.shape[0] == t.shape[0]``. The number of
       columns will be ``len(hrfs)``.
    contrasts : dict
       A contrast is generated for each HRF specified in `hrfs`.
    """
    block_spec = np.asarray(block_spec)
    if block_spec.dtype.names is not None:
        if block_spec.dtype.names not in (('start', 'end'),
                                          ('start', 'end', 'amplitude')):
            raise ValueError('expecting fields called "start", "end" and '
                             '(optionally) "amplitude"')
        block_spec = np.array(block_spec.tolist())
    block_times = block_spec[:, :2]
    amplitudes = block_spec[:, 2] if block_spec.shape[1] == 3 else None
    # Now construct the design in time space
    convolution_interval = (block_times.min() - convolution_padding,
                            block_times.max() + convolution_padding)
    B = blocks(block_times, amplitudes=amplitudes)
    t_terms = []
    c_t = {}
    n_hrfs = len(hrfs)
    for hrf_no in range(n_hrfs):
        t_terms.append(convolve_functions(B, hrfs[hrf_no](T),
                                          convolution_interval,
                                          hrf_interval,
                                          convolution_dt))
        contrast = np.zeros(n_hrfs)
        contrast[hrf_no] = 1
        c_t['{0}_{1:d}'.format(name, hrf_no)] = contrast
    t_formula = Formula(t_terms)
    tval = make_recarray(t, ['t'])
    X_t = t_formula.design(tval, return_float=True)
    return X_t, c_t



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
""" Testing fmri utils

"""
from __future__ import absolute_import, division, print_function

import re

import numpy as np

import sympy
from sympy import Symbol, Dummy, Function, DiracDelta
from sympy.utilities.lambdify import lambdify, implemented_function

from nipy.algorithms.statistics.formula import Term

from ..utils import (
    Interp1dNumeric,
    lambdify_t,
    define,
    events,
    blocks,
    interp,
    linear_interp,
    step_function,
    TimeConvolver,
    convolve_functions,
    )
from .. import hrf

from nose.tools import (assert_equal, assert_true, assert_false, raises,
                        assert_raises)

from numpy.testing import (assert_array_equal, assert_array_almost_equal,
                           assert_almost_equal)


t = Term('t')


def test_define():
    expr = sympy.exp(3*t)
    assert_equal(str(expr), 'exp(3*t)')
    newf = define('f', expr)
    assert_equal(str(newf), 'f(t)')
    f = lambdify_t(newf)
    tval = np.random.standard_normal((3,))
    assert_almost_equal(np.exp(3*tval), f(tval))


def test_events():
    # test events utility function
    h = Function('hrf')
    evs = events([3,6,9])
    assert_equal(DiracDelta(-9 + t) + DiracDelta(-6 + t) +
                 DiracDelta(-3 + t), evs)
    evs = events([3,6,9], f=h)
    assert_equal(h(-3 + t) + h(-6 + t) + h(-9 + t), evs)
    # make some beta symbols
    b = [Dummy('b%d' % i) for i in range(3)]
    a = Symbol('a')
    p = b[0] + b[1]*a + b[2]*a**2
    evs = events([3,6,9], amplitudes=[2,1,-1], g=p)
    assert_equal((2*b[1] + 4*b[2] + b[0])*DiracDelta(-3 + t) +
                 (-b[1] + b[0] + b[2])*DiracDelta(-9 + t) +
                 (b[0] + b[1] + b[2])*DiracDelta(-6 + t),
                 evs)
    evs = events([3,6,9], amplitudes=[2,1,-1], g=p, f=h)
    assert_equal((2*b[1] + 4*b[2] + b[0])*h(-3 + t) +
                 (-b[1] + b[0] + b[2])*h(-9 + t) +
                 (b[0] + b[1] + b[2])*h(-6 + t),
                 evs)
    # test no error for numpy int arrays
    onsets = np.array([30, 70, 100], dtype=np.int64)
    evs = events(onsets, f=hrf.glover)


def test_interp():
    times = [0,4,5.]
    values = [2.,4,6]
    for int_func in (interp, linear_interp):
        s = int_func(times, values, np.nan)
        tval = np.array([-0.1,0.1,3.9,4.1,5.1])
        res = lambdify(t, s)(tval)
        assert_array_equal(np.isnan(res),
                           [True, False, False, False, True])
        assert_array_almost_equal(res[1:-1], [2.05, 3.95, 4.2])
        # default is zero fill
        s = int_func(times, values)
        res = lambdify(t, s)(tval)
        assert_array_almost_equal(res, [0, 2.05, 3.95, 4.2, 0])
        # Can be some other value
        s = int_func(times, values, fill=10)
        res = lambdify(t, s)(tval)
        assert_array_almost_equal(res, [10, 2.05, 3.95, 4.2, 10])
        # If fill is None, raises error on interpolation outside bounds
        s = int_func(times, values, fill=None)
        f = lambdify(t, s)
        assert_array_almost_equal(f(tval[1:-1]), [2.05, 3.95, 4.2])
        assert_raises(ValueError, f, tval[:-1])
        # specifying kind as linear is OK
        s = linear_interp(times, values, kind='linear')
        # bounds_check should match fill
        int_func(times, values, bounds_error=False)
        int_func(times, values, fill=None, bounds_error=True)
        assert_raises(ValueError, int_func, times, values, bounds_error=True)
        # fill should match fill value
        int_func(times, values, fill=10, fill_value=10)
        int_func(times, values, fill_value=0)
        assert_raises(ValueError,
                      int_func, times, values, fill=10, fill_value=9)
        int_func(times, values, fill=np.nan, fill_value=np.nan)
        assert_raises(ValueError,
                      int_func, times, values, fill=10, fill_value=np.nan)
        assert_raises(ValueError,
                      int_func, times, values, fill=np.nan, fill_value=0)


@raises(ValueError)
def test_linear_inter_kind():
    linear_interp([0, 1], [1, 2], kind='cubic')


def test_step_function():
    # test step function
    # step function is a function of t
    s = step_function([0,4,5],[2,4,6])
    tval = np.array([-0.1,0,3.9,4,4.1,5.1])
    lam = lambdify(t, s)
    assert_array_equal(lam(tval), [0, 2, 2, 4, 4, 6])
    s = step_function([0,4,5],[4,2,1])
    lam = lambdify(t, s)
    assert_array_equal(lam(tval), [0, 4, 4, 2, 2, 1])
    # Name default
    assert_false(re.match(r'step\d+\(t\)$', str(s)) is None)
    # Name reloaded
    s = step_function([0,4,5],[4,2,1], name='goodie_goodie_yum_yum')
    assert_equal(str(s), 'goodie_goodie_yum_yum(t)')


def test_blocks():
    on_off = [[1,2],[3,4]]
    tval = np.array([0.4,1.4,2.4,3.4])
    b = blocks(on_off)
    lam = lambdify(t, b)
    assert_array_equal(lam(tval), [0, 1, 0, 1])
    b = blocks(on_off, amplitudes=[3,5])
    lam = lambdify(t, b)
    assert_array_equal(lam(tval), [0, 3, 0, 5])
    # Check what happens with names
    # Default is from step function
    assert_false(re.match(r'step\d+\(t\)$', str(b)) is None)
    # Can pass in another
    b = blocks(on_off, name='funky_chicken')
    assert_equal(str(b), 'funky_chicken(t)')


def numerical_convolve(func1, func2, interval, dt):
    mni, mxi = interval
    time = np.arange(mni, mxi, dt)
    vec1 = func1(time).astype(float)
    vec2 = func2(time).astype(float)
    value = np.convolve(vec1, vec2) * dt
    min_s = min(time.size, value.size)
    time = time[:min_s]
    value = value[:min_s]
    return time, value


def test_convolve_functions():
    # replicate convolution
    # This is a square wave on [0,1]
    f1 = (t > 0) * (t < 1)
    # ff1 is the numerical implementation of same
    ff1 = lambdify(t, f1)
    # Time delta
    dt = 1e-3
    # Numerical convolution to test against
    # The convolution of ``f1`` with itself is a triangular wave on [0, 2],
    # peaking at 1 with height 1
    time, value = numerical_convolve(ff1, ff1, [0, 2], dt)
    # shells to wrap convolve kernel version
    def kern_conv1(f1, f2, f1_interval, f2_interval, dt, fill=0, name=None):
        kern = TimeConvolver(f1, f1_interval, dt, fill)
        return kern.convolve(f2, f2_interval, name=name)
    def kern_conv2(f1, f2, f1_interval, f2_interval, dt, fill=0, name=None):
        kern = TimeConvolver(f2, f2_interval, dt, fill)
        return kern.convolve(f1, f1_interval, name=name)
    for cfunc in (convolve_functions, kern_conv1, kern_conv2):
        tri = cfunc(f1, f1, [0, 2], [0, 2], dt, name='conv')
        assert_equal(str(tri), 'conv(t)')
        ftri = lambdify(t, tri)
        y = ftri(time)
        # numerical convolve about the same as ours
        assert_array_almost_equal(value, y)
        # peak is at 1
        assert_array_almost_equal(time[np.argmax(y)], 1)
        # Flip the interval and get the same result
        for seq1, seq2 in (((0, 2), (2, 0)),
                        ((2, 0), (0, 2)),
                        ((2, 0), (2, 0))):
            tri = cfunc(f1, f1, seq1, seq2, dt)
            ftri = lambdify(t, tri)
            y = ftri(time)
            assert_array_almost_equal(value, y)
        # offset square wave by 1 - offset triangle by 1
        f2 = (t > 1) * (t < 2)
        tri = cfunc(f1, f2, [0, 3], [0, 3], dt)
        ftri = lambdify(t, tri)
        o1_time = np.arange(0, 3, dt)
        z1s = np.zeros((int(round(1./dt))))
        assert_array_almost_equal(ftri(o1_time), np.r_[z1s, value])
        # Same for input function
        tri = cfunc(f2, f1, [0, 3], [0, 3], dt)
        ftri = lambdify(t, tri)
        assert_array_almost_equal(ftri(o1_time), np.r_[z1s, value])
        # 2 seconds for both
        tri = cfunc(f2, f2, [0, 4], [0, 4], dt)
        ftri = lambdify(t, tri)
        o2_time = np.arange(0, 4, dt)
        assert_array_almost_equal(ftri(o2_time), np.r_[z1s, z1s, value])
        # offset by -0.5 - offset triangle by -0.5
        f3 = (t > -0.5) * (t < 0.5)
        tri = cfunc(f1, f3, [0, 2], [-0.5, 1.5], dt)
        ftri = lambdify(t, tri)
        o1_time = np.arange(-0.5, 1.5, dt)
        assert_array_almost_equal(ftri(o1_time), value)
        # Same for input function
        tri = cfunc(f3, f1, [-0.5, 1.5], [0, 2], dt)
        ftri = lambdify(t, tri)
        assert_array_almost_equal(ftri(o1_time), value)
        # -1 second for both
        tri = cfunc(f3, f3, [-0.5, 1.5], [-0.5, 1.5], dt)
        ftri = lambdify(t, tri)
        o2_time = np.arange(-1, 1, dt)
        assert_array_almost_equal(ftri(o2_time), value)
        # Check it's OK to be off the dt grid
        tri = cfunc(f1, f1, [dt/2, 2 + dt/2], [0, 2], dt, name='conv')
        ftri = lambdify(t, tri)
        assert_array_almost_equal(ftri(time), value, 3)
        # Check fill value
        nan_tri = cfunc(f1, f1, [0, 2], [0, 2], dt, fill=np.nan)
        nan_ftri = lambdify(t, nan_tri)
        y = nan_ftri(time)
        assert_array_equal(y, value)
        assert_true(np.all(np.isnan(nan_ftri(np.arange(-2, 0)))))
        assert_true(np.all(np.isnan(nan_ftri(np.arange(4, 6)))))
        # The original fill value was 0
        assert_array_equal(ftri(np.arange(-2, 0)), 0)
        assert_array_equal(ftri(np.arange(4, 6)), 0)


def test_interp1d_numeric():
    # Test wrapper for interp1d
    # See: https://github.com/sympy/sympy/issues/10810
    #
    # Test TypeError raised for object
    func = Interp1dNumeric(range(10), range(10))
    # Numeric values OK
    assert_almost_equal(func([1, 2, 3]), [1, 2, 3])
    assert_almost_equal(func([1.5, 2.5, 3.5]), [1.5, 2.5, 3.5])
    # Object values raise TypeError
    assert_raises(TypeError, func, t)
    # Check it works as expected via sympy
    sym_func = implemented_function('func', func)
    f = sym_func(t - 2)
    assert_almost_equal(lambdify_t(f)(4.5), 2.5)
    for val in (2, 2.):
        f = sym_func(val)
        # Input has no effect
        assert_almost_equal(lambdify_t(f)(-100), 2)
        assert_almost_equal(lambdify_t(f)(-1000), 2)
    # Float expression
    f = sym_func(t - 2.)
    assert_almost_equal(lambdify_t(f)(4.5), 2.5)


def test_convolve_hrf():
    # Check that using an HRF convolution on two events is the same as doing the
    # same convolution on each of them and summing
    t = Term('t')
    glover_t = hrf.glover(t)
    # dt is exactly representable in floating point here
    glover_conv = TimeConvolver(glover_t, [0, 26], 1./32)
    event0 = blocks(((1, 2),), (2.,))
    event1 = blocks(((15, 20),), (1.,))
    conved0 = glover_conv.convolve(event0, [0, 3])
    conved1 = glover_conv.convolve(event1, [14, 21])
    times = np.arange(0, 50, 0.1)
    e0_e1 = lambdify_t(conved0)(times) + lambdify_t(conved1)(times)
    # Same thing in one shot
    events = blocks(((1, 2), (15, 20)), (2., 1))
    conved = glover_conv.convolve(events, [0, 21])
    assert_almost_equal(lambdify_t(conved)(times), e0_e1)
    # Same thing with convolve_functions
    conved_cf = convolve_functions(events, glover_t, [0, 21], [0, 26], 1/32.)
    assert_almost_equal(lambdify_t(conved_cf)(times), e0_e1)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Utility functions for mutli-subjectParcellation:
this basically uses nipy io lib to perform IO opermation
in parcel definition processes
"""
from __future__ import print_function
from __future__ import absolute_import

import numpy as np
import os.path
from warnings import warn

from nibabel import load, save, Nifti1Image

from nipy.io.nibcompat import get_header, get_affine
from nipy.algorithms.clustering.utils import kmeans
from .discrete_domain import grid_domain_from_image
from .mroi import SubDomains
from ..mask import intersect_masks

from nipy.externals.six import string_types

warn('Module nipy.labs.spatial_models.parcel_io' + 
     'deprecated, will be removed',
     FutureWarning,
     stacklevel=2)


def mask_parcellation(mask_images, nb_parcel, threshold=0, output_image=None):
    """ Performs the parcellation of a certain mask

    Parameters
    ----------
    mask_images: string or Nifti1Image or list of strings/Nifti1Images,
                 paths of mask image(s) that define(s) the common space.
    nb_parcel: int,
               number of desired parcels
    threshold: float, optional,
               level of intersection of the masks
    output_image: string, optional
                  path of the output image

    Returns
    -------
    wim: Nifti1Imagine instance,  representing the resulting parcellation
    """
    if isinstance(mask_images, string_types):
        mask = mask_images
    elif isinstance(mask_images, Nifti1Image):
        mask = mask_images
    else:
        # mask_images should be a list
        mask_data = intersect_masks(mask_images, threshold=0) > 0
        mask = Nifti1Image(mask_data.astype('u8'),
                           get_affine(load(mask_images[0])))

    domain = grid_domain_from_image(mask)
    cent, labels, J = kmeans(domain.coord, nb_parcel)
    sub_dom = SubDomains(domain, labels)
    # get id (or labels) image
    wim = sub_dom.to_image(fid='id', roi=True)
    return wim


def parcel_input(mask_images, learning_images, ths=.5, fdim=None):
    """Instantiating a Parcel structure from a give set of input

    Parameters
    ----------
    mask_images: string or Nifti1Image or list of strings/Nifti1Images,
                 paths of mask image(s) that define(s) the common space.
    learning_images: (nb_subject-) list of  (nb_feature-) list of strings,
                     paths of feature images used as input to the
                     parcellation procedure
    ths=.5: threshold to select the regions that are common across subjects.
            if ths = .5, thethreshold is half the number of subjects
    fdim: int, optional
          if nb_feature (the dimension of the data) used in subsequent analyses
          if greater than fdim,
          a PCA is perfomed to reduce the information in the data
          Byd efault, no reduction is performed

    Returns
    -------
    domain : discrete_domain.DiscreteDomain instance
        that stores the spatial information on the parcelled domain
    feature: (nb_subect-) list of arrays of shape (domain.size, fdim)
        feature information available to parcellate the data
    """
    nb_subj = len(learning_images)

    # get a group-level mask
    if isinstance(mask_images, string_types):
        mask = mask_images
    elif isinstance(mask_images, Nifti1Image):
        mask = mask_images
    else:
        # mask_images should be a list
        grp_mask = intersect_masks(mask_images, threshold=ths) > 0
        mask = Nifti1Image(grp_mask.astype('u8'),
                           get_affine(load(mask_images[0])))

    # build the domain
    domain = grid_domain_from_image(mask, nn=6)
    #nn = 6 for speed up and stability

    # load the functional data
    feature = []
    nbeta = len(learning_images[0])
    for s in range(nb_subj):
        if len(learning_images[s]) != nbeta:
            raise ValueError('Inconsistent number of dimensions')
        feature.append(np.array([domain.make_feature_from_image(b)
                                 for b in learning_images[s]]).T)

    # Possibly reduce the dimension of the functional data
    if (len(feature[0].shape) == 1) or (fdim is None):
        return domain, feature
    if fdim < feature[0].shape[1]:
        import numpy.linalg as nl
        subj = np.concatenate([s * np.ones(feature[s].shape[0]) \
                                   for s in range(nb_subj)])
        cfeature = np.concatenate(feature)
        cfeature -= np.mean(cfeature, 0)
        m1, m2, m3 = nl.svd(cfeature, 0)
        cfeature = np.dot(m1, np.diag(m2))
        cfeature = cfeature[:, 0:fdim]
        feature = [cfeature[subj == s] for s in range(nb_subj)]

    return domain, feature


def write_parcellation_images(Pa, template_path=None, indiv_path=None,
                              subject_id=None, swd=None):
    """ Write images that describe the spatial structure of the parcellation

    Parameters
    ----------
    Pa : MultiSubjectParcellation instance,
         the description of the parcellation
    template_path: string, optional,
                   path of the group-level parcellation image
    indiv_path: list of strings, optional
                paths of the individual parcellation images
    subject_id: list of strings of length Pa.nb_subj
                subject identifiers, used to infer the paths when not available
    swd: string, optional
         output directory used to infer the paths when these are not available
    """
    # argument check
    if swd is None:
        from tempfile import mkdtemp
        swd = mkdtemp()

    if subject_id is None:
        subject_id = ['subj_%04d' % s for s in range(Pa.nb_subj)]

    if len(subject_id) != Pa.nb_subj:
        raise ValueError('subject_id does not match parcellation')

    # If necessary, generate the paths
    if template_path is None:
        template_path = os.path.join(swd, "template_parcel.nii")
    if indiv_path is None:
        indiv_path = [os.path.join(swd, "parcel%s.nii" % subject_id[s])
                        for s in range(Pa.nb_subj)]

    # write the template image
    tlabs = Pa.template_labels.astype(np.int16)
    template = SubDomains(Pa.domain, tlabs)
    template_img = template.to_image(
        fid='id', roi=True, descrip='Intra-subject parcellation template')
    save(template_img, template_path)

    # write subject-related stuff
    for s in range(Pa.nb_subj):
        # write the individual label images
        labs = Pa.individual_labels[:, s]
        parcellation = SubDomains(Pa.domain, labs)
        parcellation_img = parcellation.to_image(
            fid='id', roi=True, descrip='Intra-subject parcellation')
        save(parcellation_img, indiv_path[s])


def parcellation_based_analysis(Pa, test_images, test_id='one_sample',
                                rfx_path=None, condition_id='', swd=None):
    """ This function computes parcel averages and RFX at the parcel-level

    Parameters
    ----------
    Pa: MultiSubjectParcellation instance
        the description of the parcellation
    test_images: (Pa.nb_subj-) list of paths
                 paths of images used in the inference procedure
    test_id: string, optional,
          if test_id=='one_sample', the one_sample statstic is computed
          otherwise, the parcel-based signal averages are returned
    rfx_path: string optional,
              path of the resulting one-sample test image, if applicable
    swd: string, optional
         output directory used to compute output path if rfx_path is not given
    condition_id: string, optional,
                  contrast/condition id  used to compute output path

    Returns
    -------
    test_data: array of shape(Pa.nb_parcel, Pa.nb_subj)
               the parcel-level signal average if test is not 'one_sample'
    prfx: array of shape(Pa.nb_parcel),
          the one-sample t-value if test_id is 'one_sample'
    """
    nb_subj = Pa.nb_subj

    # 1. read the test data
    if len(test_images) != nb_subj:
        raise ValueError('Inconsistent number of test images')

    test = np.array([Pa.domain.make_feature_from_image(ti)
                     for ti in test_images]).T
    test_data = Pa.make_feature('', np.array(test))

    if test_id is not 'one_sample':
        return test_data

    # 2. perform one-sample test
    # computation
    from ..utils.reproducibility_measures import ttest
    prfx = ttest(test_data)

    # Write the stuff
    template = SubDomains(Pa.domain, Pa.template_labels)
    template.set_roi_feature('prfx', prfx)
    wim = template.to_image('prfx', roi=True)
    hdr = get_header(wim)
    hdr['descrip'] = 'parcel-based random effects image (in t-variate)'
    if rfx_path is not None:
        save(wim, rfx_path)

    return prfx


def fixed_parcellation(mask_image, betas, nbparcel, nn=6, method='ward',
                          write_dir=None, mu=10., verbose=0, fullpath=None):
    """ Fixed parcellation of a given dataset

    Parameters
    ----------
    domain/mask_image
    betas: list of paths to activation images from the subject
    nbparcel, int : number fo desired parcels
    nn=6: number of nearest neighbors  to define the image topology
          (6, 18 or 26)
    method='ward': clustering method used, to be chosen among
                   'ward', 'gkm', 'ward_and-gkm'
                   'ward': Ward's clustering algorithm
                   'gkm': Geodesic k-means algorithm, random initialization
                   'gkm_and_ward': idem, initialized by Ward's clustering
    write_di: string, topional, write directory.
                    If fullpath is None too, then no file output.
    mu = 10., float: the relative weight of anatomical information
    verbose=0: verbosity mode
    fullpath=None, string,
                   path of the output image
                   If write_dir and fullpath are None then no file output.
                   If only fullpath is None then it is the write dir + a name
                   depending on the method.

    Notes
    -----
    Ward's method takes time (about 6 minutes for a 60K voxels dataset)

    Geodesic k-means is 'quick and dirty'

    Ward's + GKM is expensive but quite good

    To reduce CPU time, rather use nn=6 (especially with Ward)
    """
    from nipy.algorithms.graph.field import field_from_coo_matrix_and_data

    if method not in ['ward', 'gkm', 'ward_and_gkm', 'kmeans']:
        raise ValueError('unknown method')
    if nn not in [6, 18, 26]:
        raise ValueError('nn should be 6,18 or 26')

    # step 1: load the data ----------------------------
    # 1.1 the domain
    domain = grid_domain_from_image(mask_image, nn)

    if method is not 'kmeans':
        # 1.2 get the main cc of the graph
        # to remove the small connected components
        pass

    coord = domain.coord

    # 1.3 read the functional data
    beta = np.array([domain.make_feature_from_image(b) for b in betas])

    if len(beta.shape) > 2:
        beta = np.squeeze(beta)

    if beta.shape[0] != domain.size:
        beta = beta.T

    feature = np.hstack((beta, mu * coord / np.std(coord)))

    #step 2: parcellate the data ---------------------------

    if method is not 'kmeans':
        g = field_from_coo_matrix_and_data(domain.topology, feature)

    if method == 'kmeans':
        _, u, _ = kmeans(feature, nbparcel)

    if method == 'ward':
        u, _ = g.ward(nbparcel)

    if method == 'gkm':
        seeds = np.argsort(np.random.rand(g.V))[:nbparcel]
        _, u, _ = g.geodesic_kmeans(seeds)

    if method == 'ward_and_gkm':
        w, _ = g.ward(nbparcel)
        _, u, _ = g.geodesic_kmeans(label=w)

    lpa = SubDomains(domain, u)

    if verbose:
        var_beta = np.array(
            [np.var(beta[lpa.label == k], 0).sum() for k in range(lpa.k)])
        var_coord = np.array(
            [np.var(coord[lpa.label == k], 0).sum() for k in range(lpa.k)])
        size = lpa.get_size()
        vf = np.dot(var_beta, size) / size.sum()
        va = np.dot(var_coord, size) / size.sum()
        print(nbparcel, "functional variance", vf, "anatomical variance", va)

    # step3:  write the resulting label image
    if fullpath is not None:
        label_image = fullpath
    elif write_dir is not None:
        label_image = os.path.join(write_dir, "parcel_%s.nii" % method)
    else:
        label_image = None

    if label_image is not None:
        lpa_img = lpa.to_image(
            fid='id', roi=True, descrip='Intra-subject parcellation image')
        save(lpa_img, label_image)
        if verbose:
            print("Wrote the parcellation images as %s" % label_image)

    return lpa



""" Testing design module
"""

from os.path import dirname, join as pjoin

import numpy as np

from ..design import (event_design, block_design, stack2designs, stack_designs,
                      openfmri2nipy, block_amplitudes)
from ..utils import (events, lambdify_t, T, convolve_functions,
                     blocks)
from ..hrf import glover, dglover
from nipy.algorithms.statistics.formula import make_recarray

from numpy.testing import (assert_almost_equal,
                           assert_array_equal)

from nose.tools import (assert_true, assert_false, assert_raises,
                        assert_equal, assert_not_equal)


THIS_DIR = dirname(__file__)


def assert_dict_almost_equal(obs, exp):
    # Check that two dictionaries with array keys are almost equal
    assert_equal(set(obs), set(exp))
    for key in exp:
        assert_almost_equal(exp[key], obs[key])


def test_event_design():
    # Test event design helper function
    # An event design with one event type
    onsets = np.array([0, 20, 40, 60])
    durations = np.array([2, 3, 4, 5])
    offsets = onsets + durations
    c_fac = np.array([1, 1, 1, 1])  # constant factor (one level)
    fac_1 = np.array([0, 1, 0, 1])  # factor 1, two levels
    fac_2 = np.array([0, 0, 1, 1])  # factor 2, two levels
    t = np.arange(0, 100, 2)

    def mk_ev_spec(factors, names):
        names = ('time',) + tuple(names)
        if len(factors) == 0:
            return make_recarray(onsets, names)
        return make_recarray(zip(onsets, *factors), names)

    def mk_blk_spec(factors, names):
        names = ('start', 'end') + tuple(names)
        return make_recarray(zip(onsets, offsets, *factors), names)

    def mk_ev_tc(ev_inds):
        # Make real time course for given event onsets
        return lambdify_t(events(onsets[ev_inds], f=glover))(t)

    def mk_blk_tc(ev_inds):
        # Make real time course for block onset / offsets
        B = blocks(zip(onsets[ev_inds], offsets[ev_inds]))
        term = convolve_functions(B, glover(T),
                                  (-5, 70),  # step func support
                                  (0, 30.),  # conv kernel support
                                  0.02)  # dt
        return lambdify_t(term)(t)

    for d_maker, spec_maker, tc_maker, null_name in (
        (event_design, mk_ev_spec, mk_ev_tc, '_event_'),
        (block_design, mk_blk_spec, mk_blk_tc, '_block_')):
        # Event spec with no event factor -> single column design, no contrasts
        spec_0 = spec_maker((), ())
        X_0, contrasts_0 = d_maker(spec_0, t)
        exp_x_0 = tc_maker(onsets==onsets)
        assert_almost_equal(X_0, exp_x_0)
        assert_dict_almost_equal(contrasts_0, dict(constant_0=1))
        X_0, contrasts_0 = d_maker(spec_0, t, level_contrasts=True)
        assert_almost_equal(X_0, exp_x_0)
        assert_dict_almost_equal(contrasts_0,
                                 {'constant_0': 1,
                                  null_name + '_1_0': 1})
        # Event spec with single factor, but only one level
        spec_1c = spec_maker((c_fac,), ('smt',))
        X_1c, contrasts_1c = d_maker(spec_1c, t)
        assert_almost_equal(X_1c, exp_x_0)
        assert_dict_almost_equal(contrasts_1c, dict(constant_0=1))
        X_1c, contrasts_1c = d_maker(spec_1c, t, level_contrasts=True)
        assert_dict_almost_equal(contrasts_1c, dict(constant_0=1, smt_1_0=1))
        # Event spec with single factor, two levels
        spec_1d = spec_maker((fac_1,), ('smt',))
        exp_x_0 = tc_maker(fac_1 == 0)
        exp_x_1 = tc_maker(fac_1 == 1)
        X_1d, contrasts_1d = d_maker(spec_1d, t)
        assert_almost_equal(X_1d, np.c_[exp_x_0, exp_x_1])
        assert_dict_almost_equal(contrasts_1d,
                                 dict(constant_0=[1, 1], smt_0=[1, -1]))
        X_1d, contrasts_1d = d_maker(spec_1d, t, level_contrasts=True)
        assert_dict_almost_equal(contrasts_1d,
                                 dict(constant_0=1,
                                      smt_0=[1, -1],  # main effect
                                      smt_0_0=[1, 0],  # level 0, hrf 0
                                      smt_1_0=[0, 1]))  # level 1, hrf 0
        # Event spec with two factors, one with two levels, another with one
        spec_2dc = spec_maker((fac_1, c_fac), ('smt', 'smte'))
        X_2dc, contrasts_2dc = d_maker(spec_2dc, t)
        assert_almost_equal(X_2dc, np.c_[exp_x_0, exp_x_1])
        assert_dict_almost_equal(contrasts_2dc,
                                {'constant_0': [1, 1],
                                'smt_0': [1, -1],  # main effect
                                'smt:smte_0': [1, -1],  # interaction
                                })
        X_2dc, contrasts_2dc = d_maker(spec_2dc, t, level_contrasts=True)
        assert_dict_almost_equal(contrasts_2dc,
                                {'constant_0': [1, 1],
                                'smt_0': [1, -1],  # main effect
                                'smt:smte_0': [1, -1],  # interaction
                                'smt_0*smte_1_0': [1, 0], # smt 0, smte 0, hrf 0
                                'smt_1*smte_1_0': [0, 1], # smt 1, smte 0, hrf 0
                                })
        # Event spec with two factors, both with two levels
        spec_2dd = spec_maker((fac_1, fac_2), ('smt', 'smte'))
        exp_x_0 = tc_maker((fac_1 == 0) & (fac_2 == 0))
        exp_x_1 = tc_maker((fac_1 == 0) & (fac_2 == 1))
        exp_x_2 = tc_maker((fac_1 == 1) & (fac_2 == 0))
        exp_x_3 = tc_maker((fac_1 == 1) & (fac_2 == 1))
        X_2dd, contrasts_2dd = d_maker(spec_2dd, t)
        assert_almost_equal(X_2dd, np.c_[exp_x_0, exp_x_1, exp_x_2, exp_x_3])
        exp_cons = {'constant_0': [1, 1, 1, 1],
                    'smt_0': [1, 1, -1, -1],  # main effect fac_1
                    'smte_0': [1, -1, 1, -1],  # main effect fac_2
                    'smt:smte_0': [1, -1, -1, 1],  # interaction
                }
        assert_dict_almost_equal(contrasts_2dd, exp_cons)
        X_2dd, contrasts_2dd = d_maker(spec_2dd, t, level_contrasts=True)
        level_cons = exp_cons.copy()
        level_cons.update({
            'smt_0*smte_0_0': [1, 0, 0, 0],  # smt 0, smte 0, hrf 0
            'smt_0*smte_1_0': [0, 1, 0, 0],  # smt 0, smte 1, hrf 0
            'smt_1*smte_0_0': [0, 0, 1, 0],  # smt 1, smte 0, hrf 0
            'smt_1*smte_1_0': [0, 0, 0, 1],  # smt 1, smte 1, hrf 0
        })
        assert_dict_almost_equal(contrasts_2dd, level_cons)
        # Test max order >> 2, no error
        X_2dd, contrasts_2dd = d_maker(spec_2dd, t, order=100)
        assert_almost_equal(X_2dd, np.c_[exp_x_0, exp_x_1, exp_x_2, exp_x_3])
        assert_dict_almost_equal(contrasts_2dd, exp_cons)
        # Test max order = 1
        X_2dd, contrasts_2dd = d_maker(spec_2dd, t, order=1)
        assert_almost_equal(X_2dd, np.c_[exp_x_0, exp_x_1, exp_x_2, exp_x_3])
        # No interaction
        assert_dict_almost_equal(contrasts_2dd,
                                {'constant_0': [1, 1, 1, 1],
                                'smt_0': [1, 1, -1, -1],  # main effect fac_1
                                'smte_0': [1, -1, 1, -1],  # main effect fac_2
                                })
    # events : test field called "time" is necessary
    spec_1d = make_recarray(zip(onsets, fac_1), ('brighteyes', 'smt'))
    assert_raises(ValueError, event_design, spec_1d, t)
    # blocks : test fields called "start" and "end" are necessary
    spec_1d = make_recarray(zip(onsets, offsets, fac_1),
                            ('mister', 'brighteyes', 'smt'))
    assert_raises(ValueError, block_design, spec_1d, t)
    spec_1d = make_recarray(zip(onsets, offsets, fac_1),
                            ('start', 'brighteyes', 'smt'))
    assert_raises(ValueError, block_design, spec_1d, t)
    spec_1d = make_recarray(zip(onsets, offsets, fac_1),
                            ('mister', 'end', 'smt'))
    assert_raises(ValueError, block_design, spec_1d, t)


def assert_des_con_equal(one, two):
    des1, con1 = one
    des2, con2 = two
    assert_array_equal(des1, des2)
    assert_equal(set(con1), set(con2))
    for key in con1:
        assert_array_equal(con1[key], con2[key])


def test_stack_designs():
    # Test stack_designs function
    N = 10
    X1 = np.ones((N, 1))
    con1 = dict(con1 = np.array([1]))
    X2 = np.eye(N)
    con2 = dict(con2 = np.array([1] + [0] * (N -1)))
    sX, sc = stack_designs((X1, con1), (X2, con2))
    X1_X2 = np.c_[X1, X2]
    exp = (X1_X2,
           dict(con1=[1] + [0] * N, con2=[0, 1] + [0] * (N - 1)))
    assert_des_con_equal((sX, sc), exp)
    # Result same when stacking just two designs
    sX, sc = stack2designs(X1, X2, {}, con2)
    # Stacking a design with empty design is OK
    assert_des_con_equal(stack2designs([], X2, con1, con2),
                         (X2, con2))
    assert_des_con_equal(stack_designs(([], con1), (X2, con2)),
                         (X2, con2))
    assert_des_con_equal(stack2designs(X1, [], con1, con2),
                         (X1, con1))
    assert_des_con_equal(stack_designs((X1, con1), ([], con2)),
                         (X1, con1))
    # Stacking one design returns output unmodified
    assert_des_con_equal(stack_designs((X1, con1)), (X1, con1))
    # Can stack design without contrasts
    assert_des_con_equal(stack_designs(X1, X2), (X1_X2, {}))
    assert_des_con_equal(stack_designs(X1, (X2, con2)),
                         (X1_X2, {'con2': [0, 1] + [0] * (N - 1)}))
    assert_des_con_equal(stack_designs((X1, con1), X2),
                         (X1_X2, {'con1': [1] + [0] * N}))
    # No-contrasts can also be 1-length tuple
    assert_des_con_equal(stack_designs((X1,), (X2, con2)),
                         (X1_X2, {'con2': [0, 1] + [0] * (N - 1)}))
    assert_des_con_equal(stack_designs((X1, con1), (X2,)),
                         (X1_X2, {'con1': [1] + [0] * N}))
    # Stack three
    X3 = np.arange(N)[:, None]
    con3 = dict(con3=np.array([1]))
    assert_des_con_equal(
        stack_designs((X1, con1), (X2, con2), (X3, con3)),
        (np.c_[X1, X2, X3],
         dict(con1=[1, 0] + [0] * N,
              con2=[0, 1] + [0] * N,
              con3=[0] * N + [0, 1])))


def test_openfmri2nipy():
    # Test loading / processing OpenFMRI stimulus file
    stim_file = pjoin(THIS_DIR, 'cond_test1.txt')
    ons_dur_amp = np.loadtxt(stim_file)
    onsets, durations, amplitudes = ons_dur_amp.T
    for in_param in (stim_file, ons_dur_amp):
        res = openfmri2nipy(in_param)
        assert_equal(res.dtype.names, ('start', 'end', 'amplitude'))
        assert_array_equal(res['start'], onsets)
        assert_array_equal(res['end'], onsets + durations)
        assert_array_equal(res['amplitude'], amplitudes)


def test_block_amplitudes():
    # Test event design helper function
    # An event design with one event type
    onsets = np.array([0, 20, 40, 60])
    durations = np.array([2, 3, 4, 5])
    offsets = onsets + durations
    amplitudes = [3, 2, 1, 4]
    t = np.arange(0, 100, 2.5)

    def mk_blk_tc(amplitudes=None, hrf=glover):
        func_amp = blocks(zip(onsets, offsets), amplitudes)
        # Make real time course for block onset / offsets / amplitudes
        term = convolve_functions(func_amp, hrf(T),
                                  (-5, 70),  # step func support
                                  (0, 30.),  # conv kernel support
                                  0.02)  # dt
        return lambdify_t(term)(t)

    no_amps = make_recarray(zip(onsets, offsets), ('start', 'end'))
    amps = make_recarray(zip(onsets, offsets, amplitudes),
                         ('start', 'end', 'amplitude'))
    X, contrasts = block_amplitudes('ev0', no_amps, t)
    assert_almost_equal(X, mk_blk_tc())
    assert_dict_almost_equal(contrasts, {'ev0_0': 1})
    # Same thing as 2D array
    X, contrasts = block_amplitudes('ev0', np.c_[onsets, offsets], t)
    assert_almost_equal(X, mk_blk_tc())
    assert_dict_almost_equal(contrasts, {'ev0_0': 1})
    # Now as list
    X, contrasts = block_amplitudes('ev0', list(zip(onsets, offsets)), t)
    assert_almost_equal(X, mk_blk_tc())
    assert_dict_almost_equal(contrasts, {'ev0_0': 1})
    # Add amplitudes
    X_a, contrasts_a = block_amplitudes('ev1', amps, t)
    assert_almost_equal(X_a, mk_blk_tc(amplitudes=amplitudes))
    assert_dict_almost_equal(contrasts_a, {'ev1_0': 1})
    # Same thing as 2D array
    X_a, contrasts_a = block_amplitudes('ev1',
                                        np.c_[onsets, offsets, amplitudes],
                                        t)
    assert_almost_equal(X_a, mk_blk_tc(amplitudes=amplitudes))
    assert_dict_almost_equal(contrasts_a, {'ev1_0': 1})
    # Add another HRF
    X_2, contrasts_2 = block_amplitudes('ev0', no_amps, t, (glover, dglover))
    assert_almost_equal(X_2, np.c_[mk_blk_tc(), mk_blk_tc(hrf=dglover)])
    assert_dict_almost_equal(contrasts_2,
                             {'ev0_0': [1, 0], 'ev0_1': [0, 1]})
    # Errors on bad input
    no_start = make_recarray(zip(onsets, offsets), ('begin', 'end'))
    assert_raises(ValueError, block_amplitudes, 'ev0', no_start, t)
    no_end = make_recarray(zip(onsets, offsets), ('start', 'finish'))
    assert_raises(ValueError, block_amplitudes, 'ev0', no_end, t)
    funny_amp = make_recarray(zip(onsets, offsets, amplitudes),
                              ('start', 'end', 'intensity'))
    assert_raises(ValueError, block_amplitudes, 'ev0', funny_amp, t)
    funny_extra = make_recarray(zip(onsets, offsets, amplitudes, onsets),
                              ('start', 'end', 'amplitude', 'extra_field'))
    assert_raises(ValueError, block_amplitudes, 'ev0', funny_extra, t)



""" Testing resample function
"""
from __future__ import absolute_import

import numpy as np

from nibabel.affines import apply_affine

from ....core.image.image_spaces import (as_xyz_image,
                                         xyz_affine)
from ....core.api import Image, vox2mni
from ..resample import resample, cast_array
from ..transform import Transform
from ..affine import Affine

from numpy.testing import assert_array_almost_equal, assert_array_equal


AUX = np.array([-1.9, -1.2, -1, 2.3, 2.9, 19, 100, 258, 258.2, 258.8, 1e5])


def test_cast_array_float():
    assert_array_equal(cast_array(AUX, np.dtype(float)), AUX)


def test_cast_array_int8():
    assert_array_equal(cast_array(AUX, np.dtype('int8')),
                       [-2, -1, -1, 2, 3, 19, 100, 127, 127, 127, 127])


def test_cast_array_uint8():
    assert_array_equal(cast_array(AUX, np.dtype('uint8')),
                       [0, 0, 0, 2, 3, 19, 100, 255, 255, 255, 255])


def test_cast_array_int16():
    assert_array_equal(cast_array(AUX, np.dtype('int16')),
                       [-2, -1, -1, 2, 3, 19, 100, 258, 258, 259, 2**15 - 1])


def test_cast_array_uint16():
    assert_array_equal(cast_array(AUX, np.dtype('uint16')),
                       [0, 0, 0, 2, 3, 19, 100, 258, 258, 259, 2**16 - 1])


def test_cast_array_int32():
    assert_array_equal(cast_array(AUX, np.dtype('int32')),
                       np.round(AUX))


def test_cast_array_uint32():
    assert_array_equal(cast_array(AUX, np.dtype('uint32')),
                       np.maximum(np.round(AUX), 0))


def _test_resample(arr, T, interp_orders):
    # Check basic cases of resampling
    img = Image(arr, vox2mni(np.eye(4)))
    for i in interp_orders:
        img2 = resample(img, T, interp_order=i)
        assert_array_almost_equal(img2.get_data(), img.get_data())
        img_aff = as_xyz_image(img)
        img2 = resample(img, T, reference=(img_aff.shape, xyz_affine(img_aff)),
                        interp_order=i)
        assert_array_almost_equal(img2.get_data(), img.get_data())


def test_resample_dtypes():
    for arr in (np.random.rand(10, 11, 12),
                np.random.randint(100, size=(10, 11, 12)) - 50):
        _test_resample(arr, Affine(), (0, 1, 3, 5))
        _test_resample(arr, Transform(lambda x : x), (0, 1, 3, 5))


class ApplyAffine(Transform):
    """ Class implements Transform protocol for testing affine Transforms
    """
    def __init__(self, aff):
        self.func = lambda pts : apply_affine(aff, pts)


def test_resample_uint_data():
    arr = np.random.randint(100, size=(10, 11, 12)).astype('uint8')
    img = Image(arr, vox2mni(np.eye(4)))
    aff_obj = Affine((.5, .5, .5, .1, .1, .1, 0, 0, 0, 0, 0, 0))
    for transform in aff_obj, ApplyAffine(aff_obj.as_affine()):
        img2 = resample(img, transform)
        assert(np.min(img2.get_data()) >= 0)
        assert(np.max(img2.get_data()) < 255)


def test_resample_outvalue():
    arr = np.arange(3*3*3).reshape(3,3,3)
    img = Image(arr, vox2mni(np.eye(4)))
    aff = np.eye(4)
    aff[0,3] = 1.
    for transform in (aff, ApplyAffine(aff)):
        for order in (1, 3):
            # Default interpolation outside is constant == 0
            img2 = resample(img, transform, interp_order=order)
            arr2 = img2.get_data()
            exp_arr = np.zeros_like(arr)
            exp_arr[:-1,:,:] = arr[1:,:,:]
            assert_array_equal(arr2, exp_arr)
            # Test explicit constant value of 0
            img2 = resample(img, transform, interp_order=order,
                            mode='constant', cval=0.)
            exp_arr = np.zeros(arr.shape)
            exp_arr[:-1, :, :] = arr[1:, :, :]
            assert_array_almost_equal(img2.get_data(), exp_arr)
            # Test constant value of 1
            img2 = resample(img, transform, interp_order=order,
                            mode='constant', cval=1.)
            exp_arr[-1, :, :] = 1
            assert_array_almost_equal(img2.get_data(), exp_arr)
            # Test nearest neighbor
            img2 = resample(img, transform, interp_order=order,
                            mode='nearest')
            exp_arr[-1, :, :] = arr[-1, :, :]
            assert_array_almost_equal(img2.get_data(), exp_arr)



""" Collection of utility functions and classes

Some of these come from the matplotlib ``cbook`` module with thanks.
"""

from functools import reduce
from operator import mul


def is_iterable(obj):
    """ Return True if `obj` is iterable
    """
    try:
        iter(obj)
    except TypeError:
        return False
    return True


def is_numlike(obj):
    """ Return True if `obj` looks like a number
    """
    try:
        obj + 1
    except:
        return False
    return True


def seq_prod(seq, initial=1):
    """ General product of sequence elements

    Parameters
    ----------
    seq : sequence
        Iterable of values to multiply.
    initial : object, optional
        Initial value

    Returns
    -------
    prod : object
        Result of ``initial * seq[0] * seq[1] .. ``.
    """
    return reduce(mul, seq, initial)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
DESCRIP = 'Read 4D image file and write 3D nifti file for each volume'
EPILOG = \
'''nipy_4dto3d will generate a series of 3D nifti images for each volume a 4D
image series in any format readable by `nibabel`.
'''
from os.path import splitext, join as pjoin, split as psplit

import nibabel as nib

from nipy.externals.argparse import (ArgumentParser,
                                     RawDescriptionHelpFormatter)


def main():
    parser = ArgumentParser(description=DESCRIP,
                            epilog=EPILOG,
                            formatter_class=RawDescriptionHelpFormatter)
    parser.add_argument('filename', type=str,
                        help='4D image filename')
    parser.add_argument('--out-path', type=str,
                        help='path for output image files')
    args = parser.parse_args()
    out_path = args.out_path
    img = nib.load(args.filename)
    imgs = nib.four_to_three(img)
    froot, ext = splitext(args.filename)
    if ext in ('.gz', '.bz2'):
        froot, ext = splitext(froot)
    if out_path is not None:
        pth, fname = psplit(froot)
        froot = pjoin(out_path, fname)
    for i, img3d in enumerate(imgs):
        fname3d = '%s_%04d.nii' % (froot, i)
        nib.save(img3d, fname3d)


if __name__ == '__main__':
    main()



""" Compatibility functions for older versions of nibabel

Nibabel <= 1.3.0 do not have these attributes:

* header
* affine
* dataobj

The equivalents for these older versions of nibabel are:

* obj.get_header()
* obj.get_affine()
* obj._data

With old nibabel, getting unscaled data used `read_img_data(img,
prefer="unscaled").  Newer nibabel should prefer the `get_unscaled` method on
the image proxy object
"""
from __future__ import absolute_import

import numpy as np

import nibabel as nib


def get_dataobj(img):
    """ Return data object for nibabel image

    Parameters
    ----------
    img : ``SpatialImage`` instance
        Instance of nibabel ``SpatialImage`` class

    Returns
    -------
    dataobj : object
        ``ArrayProxy`` or ndarray object containing data for `img`
    """
    try:
        return img.dataobj
    except AttributeError:
        return img._data


def get_header(img):
    """ Return header from nibabel image

    Parameters
    ----------
    img : ``SpatialImage`` instance
        Instance of nibabel ``SpatialImage`` class

    Returns
    -------
    header : object
        header object from `img`
    """
    try:
        return img.header
    except AttributeError:
        return img.get_header()


def get_affine(img):
    """ Return affine from nibabel image

    Parameters
    ----------
    img : ``SpatialImage`` instance
        Instance of nibabel ``SpatialImage`` class

    Returns
    -------
    affine : object
        affine object from `img`
    """
    try:
        return img.affine
    except AttributeError:
        return img.get_affine()


def get_unscaled_data(img):
    """ Get the data from a nibabel image, maybe without applying scaling

    Parameters
    ----------
    img : ``SpatialImage`` instance
        Instance of nibabel ``SpatialImage`` class

    Returns
    -------
    data : ndarray
        Data as loaded from image, not applying scaling if this can be avoided
    """
    if hasattr(nib.AnalyzeImage.ImageArrayProxy, 'get_unscaled'):
        try:
            return img.dataobj.get_unscaled()
        except AttributeError:
            return np.array(img.dataobj)
    return nib.loadsave.read_img_data(img, prefer='unscaled')



from __future__ import absolute_import
#!/usr/bin/env python
import numpy as np
import numpy.random as nr

from ..field import (field_from_coo_matrix_and_data,
                     field_from_graph_and_data)
from ..graph import wgraph_from_3d_grid

from nose.tools import assert_true, assert_equal

from numpy.testing import assert_array_equal


def basic_field(nx=10, ny=10, nz=10):
    xyz = np.reshape(np.indices((nx, ny, nz)), (3, nx * ny * nz)).T
    data = np.sum(xyz, 1).astype(np.float)
    myfield = field_from_graph_and_data(wgraph_from_3d_grid(xyz, 26), data)
    return myfield


def basic_field_random(nx=10, ny=10, nz=1):
    xyz = np.reshape(np.indices((nx, ny, nz)), (3, nx * ny * nz)).T
    data = 0.5 * nr.randn(nx * ny * nz, 1) + np.sum(xyz, 1).astype(np.float)
    myfield = field_from_graph_and_data(wgraph_from_3d_grid(xyz, 26), data)
    return myfield


def basic_field_2(nx=10, ny=10, nz=10):
    xyz = np.reshape(np.indices((nx, ny, nz)), (3, nx * ny * nz)).T
    toto = xyz - np.array([5, 5, 5])
    data = np.sum(toto ** 2, 1)
    myfield = field_from_graph_and_data(wgraph_from_3d_grid(xyz, 26), data)
    return myfield


def basic_field_3(nx=10, ny=10, nz=10):
    xyz = np.reshape(np.indices((nx, ny, nz)), (3, nx * ny * nz)).T
    toto = xyz - np.array([5, 5, 5])
    data = np.abs(np.sum(toto ** 2, 1) - 11 )
    myfield = field_from_graph_and_data(wgraph_from_3d_grid(xyz, 26), data)
    return myfield


def basic_graph(nx=10, ny=10, nz=10):
    xyz = np.reshape(np.indices((nx, ny, nz)), (3, nx * ny * nz)).T
    data = np.zeros(xyz.shape[0])
    myfield = field_from_graph_and_data(wgraph_from_3d_grid(xyz, 26), data)
    return myfield


def test_type_local_max():
    f = basic_field()
    f.field = f.field.astype(np.float32)
    idx, depth = f.get_local_maxima(th=0)
    assert_array_equal(idx, np.array([999]))


def test_max_1():
    myfield  = basic_field()
    myfield.field[555] = 30
    depth = myfield.local_maxima()
    dep = np.zeros(1000, np.int)
    dep[555] = 5
    dep[999] = 3
    assert_true(sum(np.absolute(dep-depth)) < 1.e-7)


def test_max_2():
    myfield  = basic_field()
    myfield.field[555] = 28
    idx, depth = myfield.get_local_maxima()
    assert_true(len(idx) == 2)
    assert_array_equal(idx, np.array([555, 999]))
    assert_array_equal(depth, np.array([5, 3]))


def test_max_3():
    myfield  = basic_field()
    myfield.field[555] = 27
    idx, depth = myfield.get_local_maxima()
    assert_equal(np.size(idx), 2)
    assert_equal(idx[0], 555)
    assert_equal(idx[1], 999)
    assert_equal(depth[0], 5)
    assert_equal(depth[1], 5)


def test_max_4():
    myfield  = basic_field()
    myfield.field[555] = 28
    idx, depth = myfield.get_local_maxima(0, 27.5)
    assert_equal(np.size(idx), 1)
    assert_equal(idx[0], 555)
    assert_equal(depth[0], 1)


def test_smooth_1():
    G  = basic_graph()
    field = np.zeros((1000,1))
    field[555,0] = 1
    G.set_field(field)
    G.diffusion()
    sfield = G.get_field()
    assert_equal(sfield[555], 0)
    assert_equal(sfield[554], 1)
    assert_true(np.abs(sfield[566] - np.sqrt(2)) < 1.e-7)
    assert_true(np.abs(sfield[446] - np.sqrt(3)) < 1.e-7)


def test_smooth_2():
    G  = basic_graph()
    field = np.zeros((1000, 1))
    field[555, 0] = 1
    G.set_field(field)
    G.diffusion(1)
    sfield = G.get_field()
    assert_equal(sfield[555], 0)
    assert_equal(sfield[554], 1)
    assert_true(np.abs(sfield[566] - np.sqrt(2)) < 1.e-7)
    assert_true(np.abs(sfield[446] - np.sqrt(3)) < 1.e-7)
    

def test_dilation():
    myfield  = basic_field()
    myfield.field[555] = 30
    myfield.field[664] = 0
    myfield.dilation(2)
    assert_true(myfield.field[737] == 30)
    assert_true(myfield.field[0] == 6)
    assert_true(myfield.field[999] == 27)
    assert_true(myfield.field[664] == 30)


def test_dilation2():
    # test equality of cython and python versions
    myfield  = basic_field()
    myfield.field[555] = 30
    myfield.field[664] = 0
    h = myfield.copy()
    h.dilation(2)
    g = myfield.copy()
    g.dilation(2, False)
    assert_array_equal(h.field, g.field)


def test_erosion():
    myfield  = basic_field()
    myfield.field[555] = 30
    myfield.field[664] = 0
    myfield.erosion(2)
    field = myfield.get_field()
    assert_true(field[737] == 11)
    assert_true(field[0] == 0)
    assert_true(field[999] == 21)
    assert_true(field[664] == 0)


def test_opening():
    myfield  = basic_field()
    myfield.field[555] = 30
    myfield.field[664] = 0
    myfield.opening(2)
    field = myfield.get_field()
    assert_true(field[737] == 17)
    assert_true(field[0] == 0)
    assert_true(field[999] == 21)
    assert_true(field[555] == 16)


def test_closing():
    myfield  = basic_field()
    myfield.field[555] = 30
    myfield.field[664] = 0
    myfield.closing(2)
    field = myfield.get_field()
    assert_true(field[737] == 17)
    assert_true(field[0] == 6)
    assert_true(field[999] == 27)
    assert_true(field[555] == 30)


def test_watershed_1():
    myfield = basic_field()
    myfield.field[555] = 28
    myfield.field[664] = 0
    idx, label = myfield.custom_watershed()
    assert_equal(np.size(idx), 2)
    assert_equal(tuple(idx), (555, 999))
    assert_equal((label[776], label[666], label[123]), (1, 0, 0))


def test_watershed_4():
    myfield = basic_field_3()
    idx, label = myfield.custom_watershed()
    assert_true(np.size(idx) == 9)
    assert_true(np.unique(
            [label[555], label[0], label[9], label[90], label[99], label[900],
             label[909], label[990], label[999]]).size == 9)

    
def test_watershed_2():
    myfield = basic_field_2()
    myfield.field[555] = 10
    myfield.field[664] = 0
    idx, label = myfield.custom_watershed()
    assert_true(np.size(idx) == 9)


def test_watershed_3():
    myfield  = basic_field_2()
    myfield.field[555] = 10
    myfield.field[664] = 0
    idx, label = myfield.custom_watershed(0,11)
    assert_true(np.size(idx)==8)


def test_bifurcations_1():
    myfield = basic_field()
    idx, parent,label = myfield.threshold_bifurcations()
    assert_true(idx == 999)
    assert_true(parent == 0)


def test_bifurcations_2():
    myfield = basic_field_2()
    idx, parent, label = myfield.threshold_bifurcations()
    assert_true(np.size(idx) == 15)



def test_geodesic_kmeans(nbseeds=3):
    # Test the geodisc k-means algorithm
    myfield = basic_field_random(5, 5, 1)
    seeds = np.argsort(nr.rand(myfield.V))[:nbseeds]
    seeds, label, inertia = myfield.geodesic_kmeans(seeds)
    assert_array_equal(label[seeds], np.arange(nbseeds))
    assert_true(np.array([i in np.unique(label)
                          for i in np.arange(nbseeds)]).all())


def test_constrained_voronoi(nbseeds=3):
    # Test the geodisc k-means algorithm
    myfield = basic_field_random()
    seeds = np.argsort(nr.rand(myfield.V))[:nbseeds]
    label = myfield.constrained_voronoi(seeds)
    assert_array_equal(label[seeds], np.arange(nbseeds))
    assert_true(np.array([i in np.unique(label)
                          for i in np.arange(nbseeds)]).all())


def test_constrained_voronoi_2(nbseeds=3):
    # Test the geodisc k-means algorithm
    xyz, x = np.zeros((30, 3)), np.arange(30)
    xyz[:, 0] = x
    y = np.array((x // 10), np.float)
    myfield = field_from_graph_and_data(wgraph_from_3d_grid(xyz, 6),  y)
    seeds = np.array([1, 18, 25])
    label = myfield.constrained_voronoi(seeds)
    assert_array_equal(label, x // 10)


def test_subfield():
    myfield = basic_field_random()
    valid = nr.rand(myfield.V) > 0.1
    sf = myfield.subfield(valid)
    assert_equal(sf.V, np.sum(valid))


def test_subfield2():
    myfield = basic_field_random()
    valid = np.zeros(myfield.V)
    sf = myfield.subfield(valid)
    assert_true(sf is None)


def test_ward1():
    myfield = basic_field_random()
    lab, J = myfield.ward(10)
    assert_equal(lab.max(), 9)


def test_ward2():
    myfield = basic_field_random()
    Lab, J1 = myfield.ward(5)
    Lab, J2 = myfield.ward(10)
    assert_true(J1 > J2)


def test_field_from_coo_matrix():
    import scipy.sparse as sps
    V = 10
    a = np.random.rand(V, V) > .9
    fi = field_from_coo_matrix_and_data(sps.coo_matrix(a), a)
    assert_equal(fi.E, a.sum())


if __name__ == '__main__':
    import nose
    nose.run(argv=['', __file__])



from __future__ import absolute_import
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
# init for externals package
from . import argparse
from . import configobj



#!/usr/bin/env python
from __future__ import absolute_import, print_function


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration
    config = Configuration('utils', parent_package, top_path)
    config.add_subpackage('tests')
    config.add_extension(
                'routines',
                sources=['routines.pyx'],
                libraries=['cstat']
                )

    return config


if __name__ == '__main__':
    print('This is the wrong setup.py file to run')



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
This module defines a few common generators for slicing over arrays.

They are defined on ndarray, so they do not depend on Image.

* data_generator: return (item, data[item]) tuples from an iterable object
* slice_generator: return slices through an ndarray, possibly over many
  indices
* f_generator: return a generator that applies a function to the
  output of another generator

The above three generators return 2-tuples.

* write_data: write the output of a generator to an ndarray
* parcels: return binary array of the unique components of data
"""
from __future__ import print_function
from __future__ import absolute_import

import numpy as np

from nipy.utils import seq_prod


def parcels(data, labels=None, exclude=()):
    """ Return a generator for ``[data == label for label in labels]``

    If labels is None, labels = numpy.unique(data).  Each label in labels can be
    a sequence, in which case the value returned for that label union::

        [numpy.equal(data, l) for l in label]

    Parameters
    ----------
    data : image or array-like
        Either an image (with ``get_data`` method returning ndarray) or an
        array-like
    labels : iterable, optional
        A sequence of labels for which to return indices within `data`. The
        elements in `labels` can themselves be lists, tuples, in which case the
        indices returned are for all values in `data` matching any of the items
        in the list, tuple.
    exclude : iterable, optional
        Values in `labels` for which you do not want to return a parcel.

    Returns
    -------
    gen : generator
        generator yielding a array of boolean indices into `data` for which ``data
        == label``, for each element in `label`.

    Examples
    --------
    >>> for p in parcels([[1,1],[2,1]]):
    ...     print(p)
    ...
    [[ True  True]
     [False  True]]
    [[False False]
     [ True False]]
    >>> for p in parcels([[1,1],[2,3]], labels=[2,3]):
    ...     print(p)
    ...
    [[False False]
     [ True False]]
    [[False False]
     [False  True]]
    >>> for p in parcels([[1,1],[2,3]], labels=[(2,3),2]):
    ...     print(p)
    ...
    [[False False]
     [ True  True]]
    [[False False]
     [ True False]]
    """
    # Get image data or make array from array-like
    try:
        data = data.get_data()
    except AttributeError:
        data = np.asarray(data)
    if labels is None:
        labels = np.unique(data)
    for label in labels:
        if label in exclude:
            continue
        if type(label) not in [type(()), type([])]:
            yield np.equal(data, label)
        else:
            v = 0
            for l in label:
                v += np.equal(data, l)
            yield v.astype(bool)


def data_generator(data, iterable=None):
    """ Return generator for ``[(i, data[i]) for i in iterable]``

    If iterable is None, it defaults to range(data.shape[0])

    Examples
    --------
    >>> a = np.asarray([[True,False],[False,True]])
    >>> b = np.asarray([[False,False],[True,False]])

    >>> for i, d in data_generator(np.asarray([[1,2],[3,4]]), [a,b]):
    ...     print(d)
    ...
    [1 4]
    [3]
    """
    data = np.asarray(data)
    if iterable is None:
        iterable = range(data.shape[0])
    for index in iterable:
        yield index, data[index]


def write_data(output, iterable):
    """ Write (index, data) iterable to `output`

    Write some data to `output`. Iterable should return 2-tuples of the form
    index, data such that::

        output[index] = data

    makes sense.

    Examples
    --------
    >>> a=np.zeros((2,2))
    >>> write_data(a, data_generator(np.asarray([[1,2],[3,4]])))
    >>> a
    array([[ 1.,  2.],
           [ 3.,  4.]])
    """
    for index, data in iterable:
        output[index] = data


def slice_generator(data, axis=0):
    """ Return generator for yielding slices along `axis`

    Parameters
    ----------
    data : array-like
    axis : int or list or tuple
        If int, gives the axis.  If list or tuple, gives the combination of
        axes over which to iterate.  First axis is fastest changing in output.

    Examples
    --------
    >>> for i,d in slice_generator([[1,2],[3,4]]):
    ...     print(i, d)
    ...
    (0,) [1 2]
    (1,) [3 4]
    >>> for i,d in slice_generator([[1,2],[3,4]], axis=1):
    ...     print(i, d)
    ...
    (slice(None, None, None), 0) [1 3]
    (slice(None, None, None), 1) [2 4]
    """
    data = np.asarray(data)
    if type(axis) is type(1):
        for j in range(data.shape[axis]):
            ij = (slice(None,None,None),)*axis + (j,)
            yield ij, data[(slice(None,None,None),)*axis + (j,)]
        return

    # the total number of iterations to be made
    axis_lens = [data.shape[a] for a in axis]
    nmax = seq_prod(axis_lens)

    # calculate the 'divmod' parameter which is used to work out
    # which index to use to use for each axis during iteration
    mods = np.cumprod(axis_lens)
    divs = [1] + list(mods[:-1])

    # set up a full set of slices for the image, to be modified
    # at each iteration
    slice_template = [slice(0, s) for s in data.shape]

    for n in range(nmax):
        slices = slice_template[:]
        for (a, div, mod) in zip(axis, divs, mods):
            x = int(n / div % mod)
            slices[a] = x
        yield slices, data[slices]


def f_generator(f, iterable):
    """ Return a generator for ``[(i, f(x)) for i, x in iterable]``

    Examples
    --------
    >>> for i, d in f_generator(lambda x: x**2, data_generator([[1,2],[3,4]])):
    ...     print(i, d)
    ...
    0 [1 4]
    1 [ 9 16]
    """
    for i, x in iterable:
        yield i, np.asarray(f(x))


def slice_parcels(data, labels=None, axis=0):
    """
    A generator for slicing through parcels and slices of data...

    hmmm... a better description is needed

    >>> x=np.array([[0,0,0,1],[0,1,0,1],[2,2,0,1]])
    >>> for a in slice_parcels(x):
    ...     print(a, x[a])
    ...
    ((0,), array([ True,  True,  True, False], dtype=bool)) [0 0 0]
    ((0,), array([False, False, False,  True], dtype=bool)) [1]
    ((1,), array([ True, False,  True, False], dtype=bool)) [0 0]
    ((1,), array([False,  True, False,  True], dtype=bool)) [1 1]
    ((2,), array([False, False,  True, False], dtype=bool)) [0]
    ((2,), array([False, False, False,  True], dtype=bool)) [1]
    ((2,), array([ True,  True, False, False], dtype=bool)) [2 2]
    >>> for a in slice_parcels(x, axis=1):
    ...     b, c = a
    ...     print(a, x[b][c])
    ...
    ((slice(None, None, None), 0), array([ True,  True, False], dtype=bool)) [0 0]
    ((slice(None, None, None), 0), array([False, False,  True], dtype=bool)) [2]
    ((slice(None, None, None), 1), array([ True, False, False], dtype=bool)) [0]
    ((slice(None, None, None), 1), array([False,  True, False], dtype=bool)) [1]
    ((slice(None, None, None), 1), array([False, False,  True], dtype=bool)) [2]
    ((slice(None, None, None), 2), array([ True,  True,  True], dtype=bool)) [0 0 0]
    ((slice(None, None, None), 3), array([ True,  True,  True], dtype=bool)) [1 1 1]
    """
    for i, d in slice_generator(data, axis=axis):
        for p in parcels(d, labels=labels):
            yield (i, p)


def matrix_generator(img):
    """
    From a generator of items (i, r), return
    (i, rp) where rp is a 2d array with rp.shape = (r.shape[0], prod(r.shape[1:]))
    """
    for i, r in img:
        r.shape = (r.shape[0], np.product(r.shape[1:]))
        yield i, r


def shape_generator(img, shape):
    """
    From a generator of items (i, r), return
    (i, r.reshape(shape))
    """
    for i, r in img:
        r.shape = shape
        yield i, r
